#!/usr/bin/env python
# Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
# Copyright (C) 2007-2022 German Aerospace Center (DLR) and others.
# This program and the accompanying materials are made available under the
# terms of the Eclipse Public License 2.0 which is available at
# https://www.eclipse.org/legal/epl-2.0/
# This Source Code may also be made available under the following Secondary
# Licenses when the conditions for such availability set forth in the Eclipse
# Public License 2.0 are satisfied: GNU General Public License, version 2
# or later which is available at
# https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
# SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later

# @file    runner.py
# @author  Daniel Krajzewicz
# @date    2007-07-26


from __future__ import absolute_import
from __future__ import print_function
import sys
import os
import subprocess
import random
import shutil

random.seed(42)
if "SUMO_HOME" in os.environ:
    sys.path.append(os.path.join(os.environ["SUMO_HOME"], "tools"))
import sumolib  # noqa

types = ["static", "actuated", "sotl_phase", "sotl_platoon", "sotl_request", "sotl_wave", "sotl_marching", "swarm"]
tlType = sys.argv[1]
flow1def = "0;2000;600".split(";")
flow2def = "0;2000;600".split(";")
fillSteps = 120  # 3600
measureSteps = 600  # 36000
simSteps = fillSteps + measureSteps


def buildDemand(simSteps, pWE, pEW, pNS, pSN):
    fd = open("input_routes.rou.xml", "w")
    # ---routes---
    print("""<routes>

            <vType id="type1" accel="2.0" decel="5.0" sigma="0.0" length="6.5" maxSpeed="70"/>

            <route id="WE" edges="1i 3o 5o"/>
            <route id="NS" edges="2i 4o 6o"/>
            <route id="EW" edges="3i 1o 7o"/>
            <route id="SN" edges="4i 2o 8o"/>

        """, file=fd)
    vehNr = 0
    for i in range(simSteps):
        if random.uniform(0, 1) < pWE:  # Poisson distribution
            print('    <vehicle id="%i" type="type1" route="WE" depart="%i" departSpeed="13.89" />' % (
                vehNr, i), file=fd)
            vehNr += 1
        if random.uniform(0, 1) < pNS:
            print('    <vehicle id="%i" type="type1" route="NS" depart="%i" departSpeed="13.89" />' % (
                vehNr, i), file=fd)
            vehNr += 1
        if random.uniform(0, 1) < pEW:
            print('    <vehicle id="%i" type="type1" route="EW" depart="%i" departSpeed="13.89" />' % (
                vehNr, i), file=fd)
            vehNr += 1
        if random.uniform(0, 1) < pSN:
            print('    <vehicle id="%i" type="type1" route="SN" depart="%i" departSpeed="13.89" />' % (
                vehNr, i), file=fd)
            vehNr += 1
    print("</routes>", file=fd)
    fd.close()


def patchTLSType(ifile, itype, ofile, otype):
    fdi = open(ifile)
    fdo = open(ofile, "w")
    for line in fdi:
        line = line.replace(itype, otype)
        fdo.write(line)
    fdo.close()
    fdi.close()


def main():
    try:
        os.mkdir("results")
    except OSError:
        pass
    try:
        os.mkdir("gfx")
    except OSError:
        pass

    sumo = sumolib.checkBinary('sumo')

    for f1 in range(int(flow1def[0]), int(flow1def[1]), int(flow1def[2])):
        pWE = float(f1) / 3600  # [veh/s]
        pEW = pWE
        for f2 in range(int(flow2def[0]), int(flow2def[1]), int(flow2def[2])):
            pNS = float(f2) / 3600  # [veh/s]
            pSN = pNS
            print("Computing for %s<->%s" % (f1, f2))
            buildDemand(simSteps, pWE, pEW, pNS, pSN)
            print(" for tls-type %s" % tlType)
            patchTLSType('input_additional_template.add.xml',
                         '%tls_type%', 'input_additional.add.xml', tlType)
            args = [sumo,
                    '--no-step-log',
                    # '--no-duration-log',
                    # '--verbose',
                    # '--duration-log.statistics',
                    '--default.speeddev', '0',
                    '--net-file', 'input_net.net.xml',
                    '--route-files', 'input_routes.rou.xml',
                    '--additional-files', 'input_additional.add.xml',
                    '--tripinfo-output', 'results/tripinfos_%s_%s_%s.xml' % (
                        tlType, f1, f2),
                    '--summary-output', 'results/summary_%s_%s_%s.xml' % (
                        tlType, f1, f2),
                    '--device.emissions.probability', '1',
                    '--queue-output', 'results/queue_%s_%s_%s.xml' % (
                        tlType, f1, f2),
                    '--statistic-output', 'statistics.xml',
                    ]
            subprocess.call(args)
            shutil.move(
                "results/e2_output.xml", "results/e2_output_%s_%s_%s.xml" %
                (tlType, f1, f2))
            shutil.move("results/e2_tl0_output.xml",
                        "results/e2_tl0_output_%s_%s_%s.xml" % (tlType, f1, f2))
            shutil.move("results/edgeData_3600.xml",
                        "results/edgeData_3600_%s_%s_%s.xml" % (tlType, f1, f2))
            shutil.move("results/laneData_3600.xml",
                        "results/laneData_3600_%s_%s_%s.xml" % (tlType, f1, f2))
            shutil.move("results/edgesEmissions_3600.xml",
                        "results/edgesEmissions_3600_%s_%s_%s.xml" % (tlType, f1, f2))
            shutil.move("results/lanesEmissions_3600.xml",
                        "results/lanesEmissions_3600_%s_%s_%s.xml" % (tlType, f1, f2))
            shutil.move(
                "results/TLSStates.xml", "results/TLSStates_%s_%s_%s.xml" %
                (tlType, f1, f2))
            shutil.move("results/TLSSwitchTimes.xml",
                        "results/TLSSwitchTimes_%s_%s_%s.xml" % (tlType, f1, f2))
            shutil.move("results/TLSSwitchStates.xml",
                        "tls_state.xml")


if __name__ == "__main__":
    main()



#!/usr/bin/env python
# Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
# Copyright (C) 2008-2022 German Aerospace Center (DLR) and others.
# This program and the accompanying materials are made available under the
# terms of the Eclipse Public License 2.0 which is available at
# https://www.eclipse.org/legal/epl-2.0/
# This Source Code may also be made available under the following Secondary
# Licenses when the conditions for such availability set forth in the Eclipse
# Public License 2.0 are satisfied: GNU General Public License, version 2
# or later which is available at
# https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
# SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later

# @file    runner.py
# @author  Daniel Krajzewicz
# @author  Michael Behrisch
# @author  Jakob Erdmann
# @date    2012-08-10

from __future__ import absolute_import
from __future__ import print_function

import os
import subprocess
import sys
sys.path.append(os.path.join(
    os.path.dirname(sys.argv[0]), '..', '..', '..', '..', '..', "tools"))
import traci  # noqa
import sumolib  # noqa


PORT = 8813
DELTA_T = 1000

netconvertBinary = sumolib.checkBinary('netconvert')
sumoBinary = sumolib.checkBinary('sumo')


edges = [
    ["beg", ['<route id="left2end" edges="beg beg2left left left2end end"/>',
             '<route id="right2end" edges="beg beg2right right right2end end"/>']],
    ["preBeg", ['<route id="left2end" edges="preBeg beg beg2left left left2end end"/>',
                '<route id="right2end" edges="preBeg beg beg2right right right2end end"/>']]
]

rerouter = [
    ["closing", '<closingReroute id="middle"/>'],
    ["destProb", '<destProbReroute id="left2end" probability=".5"/>',
     '<destProbReroute id="right2end" probability=".5"/>'],
    ["routeProb",  '<routeProbReroute id="left2end" probability=".5"/>',
     '<routeProbReroute id="right2end" probability=".5"/>']
]

times = [
    # times for rerouter actions - begin of simulation, begin of rerouter
    # working, end of rerouter working
    [0, 0, 86400],
    [0, 100, 86400],
    [100, 0, 86400],
    [0, 50, 100],
    [50, 50, 100],
    [100, 50, 100]
]


def writeRerouterDefinition(fdo, edge, t, rerouter):
    fdo.write('    <interval begin="%s" end="%s">\n' % (t[1], t[2]))
    for i in range(1, len(rerouter)):
        fdo.write('        %s\n' % rerouter[i].replace("%e", edge))
    fdo.write('    </interval>\n')


def writeRerouter(edge, t, rerouter, embedded):
    fdo = open("rerouter.xml", "w")
    fdo.write('<?xml version="1.0" encoding="UTF-8"?>\n')
    fdo.write('<additional xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"'
              + ' xsi:noNamespaceSchemaLocation="http://sumo.dlr.de/xsd/additional_file.xsd">\n')

    definition_file = "input_definition.def.xml"
    if embedded:
        fdo.write('<rerouter id="rerouter" edges="%s">\n' % (edge))
        writeRerouterDefinition(fdo, edge, t, rerouter)
        fdo.write('</rerouter>\n')
    else:
        fdo.write('<rerouter id="rerouter" edges="%s">\n' % (edge))
        fdo.write('  <include href="%s"/>\n' % definition_file)
        fdo.write('</rerouter>\n')

    fdo.write('</additional>\n')
    fdo.close()
    if not embedded:
        fdo = open(definition_file, "w")
        writeRerouterDefinition(fdo, edge, t, rerouter)
        fdo.close()


def writeRoutes(routes, multiRef):
    fdo = open("input_routes.rou.xml", "w")
    fdo.write(
        '<routes xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:noNamespaceSchemaLocation=' +
        '"http://sumo.dlr.de/xsd/routes_file.xsd">\n\n')
    fdo.write(
        '    <vType id="DEFAULT_VEHTYPE" accel="2.6" decel="4.5" sigma="0" length="3" minGap="2" maxSpeed="70"/>\n')
    for r in routes:
        fdo.write('    ' + r + '\n')
    if multiRef:
        fdo.write(
            '    <route id="base" edges="preBeg beg middle end rend"/>\n')
    fdo.write('\n')
    for i in range(0, 10):
        if multiRef:
            fdo.write(
                '    <vehicle id="%s" depart="%s" departPos="0" route="base"/>\n' % (i, i * 20))
        else:
            fdo.write(
                '    <vehicle id="%s" depart="%s" departPos="0">\n' % (i, i * 20))
            fdo.write('        <route edges="preBeg beg middle end rend"/>\n')
            fdo.write('    </vehicle>\n')
    fdo.write('</routes>\n')
    fdo.close()


def getTimeOnEdge(route, exitTimes, edge, depart):
    i = route.index(edge)
    if i == 0:
        return depart, float(exitTimes[0])
    else:
        return float(exitTimes[i - 1]), float(exitTimes[i])


def verify(vehroutes, edge):
    for v in vehroutes:
        if v.routeDistribution is None:
            wasRerouted = False
            entryTime, leaveTime = getTimeOnEdge(
                v.route[0].edges.split(), v.route[0].exitTimes.split(), edge, float(v.depart))
        else:
            wasRerouted = True
            fr = v.routeDistribution[0].route[-1]
            entryTime, leaveTime = getTimeOnEdge(
                fr.edges.split(), fr.exitTimes.split(), edge, float(v.depart))
        if entryTime >= t[1] and entryTime < t[2] and not wasRerouted:
            print("Vehicle '%s' entering at %s was not rerouted; times (%s, %s, %s)" % (
                v.id, entryTime, t[0], t[1], t[2]))
            return
        if wasRerouted and (entryTime < t[1] and entryTime >= t[2]):
            print("Vehicle '%s' entering at %s was rerouted though the rerouter was off; times (%s, %s, %s)" % (
                v.id, entryTime, t[0], t[1], t[2]))


print(">>> Building the network")
sys.stdout.flush()
retcode = subprocess.call(
    [netconvertBinary, "-c", "netconvert.netccfg"], stdout=sys.stdout, stderr=sys.stderr)
sys.stdout.flush()

for r in rerouter:
    nd = open(os.path.devnull)
    print("Checking %s rerouting" % r[0])
    for edge in edges:
        print(" Checking at edge '%s'" % (edge[0]))
        for t in times:
            print("  Checking for simulation begin=%s, rerouter starts at %s, ends at %s" % (
                t[0], t[1], t[2]))
            for multi in [True, False]:
                print("    Checking with multi-referenced routes: %s" % multi)
                for emb in [True, False]:
                    print(
                        "     Checking with embedded rerouter definition: %s" % emb)
                    writeRoutes(edge[1], multi)
                    writeRerouter(edge[0], t, r, emb)

                    sys.stdout.flush()
                    retcode = subprocess.call(
                        [sumoBinary, "-c", "sumo.sumocfg", "-b", str(t[0])], stdout=nd, stderr=sys.stderr)
                    sys.stdout.flush()
                    v = {}

                    vehroutes = list(sumolib.output.parse(
                        "vehroutes.xml", ["vehicle"], {'route': ['exitTimes', 'edges']}))

                    wvn = 10 - int((t[0] + 10) / 20)
                    if len(vehroutes) != wvn:
                        print(
                            "Mismatching number of vehicles (%s instead of %s)" % (len(vehroutes), wvn))

                    verify(vehroutes, edge[0])
                    try:
                        os.remove("vehroutes.xml")
                    except OSError:
                        pass
    nd.close()



#!/usr/bin/env python
# Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
# Copyright (C) 2008-2022 German Aerospace Center (DLR) and others.
# This program and the accompanying materials are made available under the
# terms of the Eclipse Public License 2.0 which is available at
# https://www.eclipse.org/legal/epl-2.0/
# This Source Code may also be made available under the following Secondary
# Licenses when the conditions for such availability set forth in the Eclipse
# Public License 2.0 are satisfied: GNU General Public License, version 2
# or later which is available at
# https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
# SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later

# @file    runner.py
# @author  Daniel Krajzewicz
# @author  Michael Behrisch
# @date    2012-10-19

from __future__ import absolute_import
from __future__ import print_function

import os
import sys

sumoHome = os.path.abspath(os.path.join(os.path.dirname(sys.argv[0]), '..', '..', '..', '..', '..', '..'))
sys.path.append(os.path.join(sumoHome, "tools"))
import sumolib  # noqa
import traci  # noqa


if sys.argv[1] == "sumo":
    sumoCall = [sumolib.checkBinary('sumo')]
else:
    sumoCall = [sumolib.checkBinary('sumo-gui'), '-S', '-Q']


def runSingle(traciEndTime, laneIndex, vehID):
    step = 0
    traci.start(sumoCall + ["-c", "sumo.sumocfg"])

    # disable safety checks and set constant speed
    print("old lanechangemode", format(traci.vehicle.getLaneChangeMode(vehID), '012b'))
    print("old speedmode", traci.vehicle.getSpeedMode(vehID))
    for veh in ("collider", "left", "right"):
        traci.vehicle.setLaneChangeMode(veh, 0b010001010101)
        traci.vehicle.setSpeedMode(veh, 0)
        traci.vehicle.setSpeed(veh, 20)
    print("new lanechangemode", format(traci.vehicle.getLaneChangeMode(vehID), '012b'))
    print("new speedmode", traci.vehicle.getSpeedMode(vehID))

    while not step > traciEndTime:
        traci.simulationStep()

        if step == 5:
            print("trying to change lane...")
            traci.vehicle.changeLane(vehID, laneIndex, traciEndTime)

        # check if vehID has arrived at destination
        arrivedList = traci.simulation.getArrivedIDList()
        if vehID in arrivedList:
            print("[%03d] Vehicle '%s' has arrived at destination" % (step, vehID))
            break

        print("[%03d] lane %d, lateral pos: %.2f" %
              (step, traci.vehicle.getLaneIndex(vehID), traci.vehicle.getLateralLanePosition(vehID)))
        sys.stdout.flush()

        step += 1

    print("Print ended at step %s" % traci.simulation.getTime())
    traci.close()
    sys.stdout.flush()


sys.stdout.flush()
sys.stderr.flush()
runSingle(50, int(sys.argv[2]), "collider")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
# Copyright (C) 2008-2022 German Aerospace Center (DLR) and others.
# This program and the accompanying materials are made available under the
# terms of the Eclipse Public License 2.0 which is available at
# https://www.eclipse.org/legal/epl-2.0/
# This Source Code may also be made available under the following Secondary
# Licenses when the conditions for such availability set forth in the Eclipse
# Public License 2.0 are satisfied: GNU General Public License, version 2
# or later which is available at
# https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
# SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later

# @file    runner.py
# @author  Jakob Erdmann
# @date    2017-01-23


from __future__ import print_function
from __future__ import absolute_import
import os
import sys

if "SUMO_HOME" in os.environ:
    sys.path.append(os.path.join(os.environ['SUMO_HOME'], 'tools'))
import traci  # noqa
import sumolib  # noqa

traci.start([sumolib.checkBinary('sumo'), '--device.rerouting.adaptation-steps', '20', '-c', 'sumo.sumocfg'])
traci.simulationStep()
while traci.simulation.getMinExpectedNumber() > 0:
    timeS = traci.simulation.getTime()
    for vehID in traci.simulation.getDepartedIDList():
        if "custom" in sys.argv:
            traci.vehicle.setAdaptedTraveltime(vehID, "detour", 1)
        if "raw" in sys.argv:
            traci.vehicle.rerouteTraveltime(vehID, False)
        elif "aggregated" in sys.argv:
            traci.vehicle.setRoutingMode(vehID, traci.constants.ROUTING_MODE_AGGREGATED)
            traci.vehicle.rerouteTraveltime(vehID)
        else:
            traci.vehicle.rerouteTraveltime(vehID)
        print("step=%s rerouted=%s" % (timeS, vehID))
    # print("step=%s", timeS)
    # for edgeID in traci.edge.getIDList():
    #    print("   edge=%s tt=%s" % (edgeID,
    #        traci.edge.getAdaptedTraveltime(edgeID, timeS)))
    traci.simulationStep()
traci.close()



#!/usr/bin/env python
# Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
# Copyright (C) 2008-2022 German Aerospace Center (DLR) and others.
# This program and the accompanying materials are made available under the
# terms of the Eclipse Public License 2.0 which is available at
# https://www.eclipse.org/legal/epl-2.0/
# This Source Code may also be made available under the following Secondary
# Licenses when the conditions for such availability set forth in the Eclipse
# Public License 2.0 are satisfied: GNU General Public License, version 2
# or later which is available at
# https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
# SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later

# @file    runner.py
# @author  Leonhard Luecken
# @date    2017-10-16

from __future__ import absolute_import
from __future__ import print_function

import os
import subprocess
import sys

if "SUMO_HOME" in os.environ:
    sys.path.append(os.path.join(os.environ["SUMO_HOME"], "tools"))

import traci  # noqa
import sumolib  # noqa

sumoBinary = sumolib.checkBinary('sumo')
subprocess.call([sumoBinary, "save_state.sumocfg"])
# LOAD
loadParams = ["-c", "sumo.sumocfg"]
print("Starting...")
print("loadParams: %s" % str(loadParams))
v = traci.start([sumoBinary] + loadParams)
while traci.simulation.getTime() < 300:
    traci.simulationStep()
print(traci.simulation.getTime())
traci.close()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
# Copyright (C) 2008-2022 German Aerospace Center (DLR) and others.
# This program and the accompanying materials are made available under the
# terms of the Eclipse Public License 2.0 which is available at
# https://www.eclipse.org/legal/epl-2.0/
# This Source Code may also be made available under the following Secondary
# Licenses when the conditions for such availability set forth in the Eclipse
# Public License 2.0 are satisfied: GNU General Public License, version 2
# or later which is available at
# https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
# SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later

# @file    runner.py
# @author  Michael Behrisch
# @author  Daniel Krajzewicz
# @date    2011-03-04


from __future__ import print_function
from __future__ import absolute_import
import os
import sys

if "SUMO_HOME" in os.environ:
    sys.path.append(os.path.join(os.environ["SUMO_HOME"], "tools"))
import traci  # noqa
import sumolib  # noqa


def examine(polygonID):
    print("# examining", polygonID)
    print("shape", traci.polygon.getShape(polygonID))
    print("type", traci.polygon.getType(polygonID))
    print("color", traci.polygon.getColor(polygonID))
    print("filled", traci.polygon.getFilled(polygonID))


traci.start([sumolib.checkBinary('sumo'), "-c", "sumo.sumocfg"])
# traci.init(port=12345) # debug
for step in range(3):
    print("step", step)
    traci.simulationStep()
polygonID = "poly0"
print("adding", polygonID)
traci.polygon.add(
    polygonID, ((960, 510), (960, 550), (1000, 550), (1000, 510)), (100, 200, 0, 255), True, "test")

print("polygons", traci.polygon.getIDList())
print("polygon count", traci.polygon.getIDCount())
examine(polygonID)

for step in range(3, 6):
    print("step", step)
    traci.simulationStep()

# Failing specification tests
print("# (1) Adding underspecified dynamics...")
try:
    traci.polygon.addDynamics(polygonID)
except traci.exceptions.TraCIException:
    print("Caught TraCIException")

print("# (2) Adding malformed dynamics 1 ...")
try:
    traci.polygon.addDynamics(polygonID, "", [0, 1, 2, 4, 3])
except traci.exceptions.TraCIException:
    print("Caught TraCIException")

print("# (3) Adding malformed dynamics 2 ...")
try:
    traci.polygon.addDynamics(polygonID, "", [1, 2, 3, 4], [200, 20, 2, 1])
except traci.exceptions.TraCIException:
    print("Caught TraCIException")

print("# (4) Adding malformed dynamics 3 ...")
try:
    traci.polygon.addDynamics(polygonID, "", [0, 1, 2, 3], [200, 20, 2])
except traci.exceptions.TraCIException:
    print("Caught TraCIException")

print("# (5) Adding malformed dynamics 4 ...")
try:
    traci.polygon.addDynamics(polygonID, "", [0], [200])
except traci.exceptions.TraCIException:
    print("Caught TraCIException")

print("# (6) Adding malformed dynamics 5 ...")
try:
    traci.polygon.addDynamics(polygonID, "horiz", [], [], True)
except traci.exceptions.TraCIException:
    print("Caught TraCIException")

examine(polygonID)

for step in range(6, 9):
    print("step", step)
    traci.simulationStep()

# Successful specifications

print("# (1) No tracking with alpha animation")
traci.polygon.addDynamics(polygonID, "", [0, 1, 2, 5], [0, 200, 100, 0])

for step in range(9, 12):
    print("step", step)
    traci.simulationStep()
    examine(polygonID)

print("polygons", traci.polygon.getIDList())
print("polygon count", traci.polygon.getIDCount())

print("vehicles", traci.vehicle.getIDList())
print("vehicle count", traci.vehicle.getIDCount())

for step in range(12, 15):
    print("step", step)
    traci.simulationStep()

print("polygons", traci.polygon.getIDList())
print("polygon count", traci.polygon.getIDCount())

polygonID = "poly1"
print("adding", polygonID)
traci.polygon.add(
    polygonID, ((960, 510), (960, 550), (1000, 550), (1000, 510)), (0, 100, 100, 255), True, "test")

print("polygons", traci.polygon.getIDList())
print("polygon count", traci.polygon.getIDCount())

print("# (2) No tracking with time line but no animation (removal after time line expired)")
traci.polygon.addDynamics(polygonID, "", [0, 1, 2, 3])

for step in range(15, 18):
    print("step", step)
    traci.simulationStep()

print("polygons", traci.polygon.getIDList())
print("polygon count", traci.polygon.getIDCount())

polygonID = "poly2"
print("adding", polygonID)
traci.polygon.add(
    polygonID, ((960, 510), (960, 550), (1000, 550), (1000, 510)), (150, 0, 100, 255), True, "test")

vehID = "veh0"
print("adding", vehID)
traci.route.add("trip0", ["2fi", "1o"])
traci.vehicle.add(vehID, "trip0")

print("vehicles", traci.vehicle.getIDList())
print("vehicle count", traci.vehicle.getIDCount())

print("# (3) Tracking without time line")
traci.polygon.addDynamics(polygonID, vehID)

for step in range(18, 21):
    print("step", step)
    traci.simulationStep()

print("# (4) Tracking existing vehicle with time line")
polygonID = "poly3"
vehID = "horiz"
traci.polygon.add(
    polygonID, ((750, 490), (750, 450), (790, 450), (790, 490)), (0, 0, 200, 255), True, "test")
traci.polygon.addDynamics(polygonID, vehID, [0, 10])

print("polygons", traci.polygon.getIDList())
print("polygon count", traci.polygon.getIDCount())

for step in range(21, 25):
    print("step", step)
    traci.simulationStep()

print("# (5) replacing tracking dynamics and highlighting vehicle permanently")
polygonID = "poly2"
vehID = "veh0"
traci.polygon.addDynamics(polygonID, vehID, [0, 2, 4, 6, 8, 10, 12, 14, 16, 18, 20], [
                          50, 200, 50, 200, 50, 200, 50, 200, 50, 200, 50])
traci.vehicle.highlight(vehID)

print("polygons", traci.polygon.getIDList())
print("polygon count", traci.polygon.getIDCount())

for step in range(25, 35):
    print("step", step)
    traci.simulationStep()

# "poly3" should have been removed
print("polygons", traci.polygon.getIDList())
print("polygon count", traci.polygon.getIDCount())

print("# (6) Tracking with alpha animation")
vehID = "horiz"
polygonID = "poly4"
traci.polygon.add(
    polygonID, ((550, 510), (550, 550), (590, 550), (590, 510)), (20, 20, 200, 255), True, "test")
traci.polygon.addDynamics(polygonID, vehID, [0, 2, 4, 6, 8, 10, 12, 14, 16, 18, 20], [
                          50, 200, 50, 200, 50, 200, 50, 200, 50, 200, 50])

for step in range(35, 40):
    print("step", step)
    traci.simulationStep()

print("# (7) Add a second tracking polygon")
vehID = "horiz"
polygonID = "poly5"
traci.polygon.add(
    polygonID, ((350, 490), (350, 450), (390, 450), (390, 490)), (200, 20, 20, 255), True, "test")
traci.polygon.addDynamics(polygonID, vehID)

print("polygons", traci.polygon.getIDList())
print("polygon count", traci.polygon.getIDCount())

for step in range(40, 45):
    print("step", step)
    traci.simulationStep()

print("polygons", traci.polygon.getIDList())
print("polygon count", traci.polygon.getIDCount())

print("# (8) Parking vehicle and renew dynamics and highlight")
vehID = "veh0"
polygonID = "poly2"
traci.vehicle.setStop(vehID, "2si", 100, 1, 5, 1)
traci.polygon.addDynamics(polygonID, vehID, [0, 2, 4, 6, 8, 10, 12, 14, 16, 18, 20], [
                          50, 200, 50, 200, 50, 200, 50, 200, 50, 200, 50])
traci.vehicle.highlight(vehID, (0, 100, 100, 100))

for step in range(45, 65):
    print("step", step)
    traci.simulationStep()

print("polygons", traci.polygon.getIDList())
print("polygon count", traci.polygon.getIDCount())

for step in range(65, 110):
    print("step", step)
    traci.simulationStep()

print("# (9) transfer tracking dynamics from vehicle 'horiz' to 'veh0' and redefine as looped animation")
vehID = "veh0"
polygonID = "poly5"
traci.polygon.addDynamics(polygonID, vehID, [0, 2, 4], [50, 200, 50], True)

print("# (10) Highlight vehicle 'horiz' for limited duration")
vehID = "horiz"
traci.vehicle.highlight(vehID, (0, 255, 0), -1, 200, 4)

print("polygons", traci.polygon.getIDList())
print("polygon count", traci.polygon.getIDCount())

for pID in traci.polygon.getIDList():
    examine(pID)

print("vehicles", traci.vehicle.getIDList())
print("vehicle count", traci.vehicle.getIDCount())

for step in range(110, 120):
    print("step", step)
    traci.simulationStep()

print("# (11) Tracked vehicle 'veh0' arrives")
for step in range(120, 125):
    print("step", step)
    traci.simulationStep()

print("polygons", traci.polygon.getIDList())
print("polygon count", traci.polygon.getIDCount())

print("vehicles", traci.vehicle.getIDList())
print("vehicle count", traci.vehicle.getIDCount())

print("# (12) Adding and highlighting POIs (two highlights for the same must specify highlight type)")

print("pois", traci.poi.getIDList())
print("poi count", traci.poi.getIDCount())

print("polygons", traci.polygon.getIDList())
print("polygon count", traci.polygon.getIDCount())

poiID = "myPOI0"
print("adding", poiID)
traci.poi.add(poiID, 350, 510, (255, 255, 255, 255), "test", 0, "img.jpeg")
traci.poi.highlight(poiID, (0, 0, 255), -1, 200, 4)

poiID = "myPOI1"
print("adding", poiID)
traci.poi.add(poiID, 370, 510, (255, 255, 255, 255), "test", 0, "img.jpeg")
traci.poi.highlight(poiID, (0, 0, 255), -1, 200, 4)

poiID = "myPOI0"
traci.poi.highlight(poiID, (55, 55, 100), 20, 200, 4, 1)

print("pois", traci.poi.getIDList())
print("poi count", traci.poi.getIDCount())

print("polygons", traci.polygon.getIDList())
print("polygon count", traci.polygon.getIDCount())

for pID in traci.polygon.getIDList():
    examine(pID)

for step in range(125, 130):
    print("step", step)
    traci.simulationStep()

print("pois", traci.poi.getIDList())
print("poi count", traci.poi.getIDCount())

print("polygons", traci.polygon.getIDList())
print("polygon count", traci.polygon.getIDCount())

for pID in traci.polygon.getIDList():
    examine(pID)

traci.close()



#!/usr/bin/env python
# Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
# Copyright (C) 2009-2022 German Aerospace Center (DLR) and others.
# This program and the accompanying materials are made available under the
# terms of the Eclipse Public License 2.0 which is available at
# https://www.eclipse.org/legal/epl-2.0/
# This Source Code may also be made available under the following Secondary
# Licenses when the conditions for such availability set forth in the Eclipse
# Public License 2.0 are satisfied: GNU General Public License, version 2
# or later which is available at
# https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
# SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later

# @file    runner.py
# @author  Lena Kalleske
# @author  Daniel Krajzewicz
# @author  Michael Behrisch
# @author  Jakob Erdmann
# @date    2009-03-26

from __future__ import absolute_import
from __future__ import print_function

import os
import sys
import optparse
import random

# we need to import python modules from the $SUMO_HOME/tools directory
if 'SUMO_HOME' in os.environ:
    tools = os.path.join(os.environ['SUMO_HOME'], 'tools')
    sys.path.append(tools)
else:
    sys.exit("please declare environment variable 'SUMO_HOME'")

from sumolib import checkBinary  # noqa
import traci  # noqa


def generate_routefile():
    random.seed(42)  # make tests reproducible
    N = 3600  # number of time steps
    # demand per second from different directions
    pWE = 1. / 10
    pEW = 1. / 11
    pNS = 1. / 30
    with open("data/cross.rou.xml", "w") as routes:
        print("""<routes>
        <vType id="typeWE" accel="0.8" decel="4.5" sigma="0.5" length="5" minGap="2.5" maxSpeed="16.67" \
guiShape="passenger"/>
        <vType id="typeNS" accel="0.8" decel="4.5" sigma="0.5" length="7" minGap="3" maxSpeed="25" guiShape="bus"/>

        <route id="right" edges="51o 1i 2o 52i" />
        <route id="left" edges="52o 2i 1o 51i" />
        <route id="down" edges="54o 4i 3o 53i" />""", file=routes)
        vehNr = 0
        for i in range(N):
            if random.uniform(0, 1) < pWE:
                print('    <vehicle id="right_%i" type="typeWE" route="right" depart="%i" />' % (
                    vehNr, i), file=routes)
                vehNr += 1
            if random.uniform(0, 1) < pEW:
                print('    <vehicle id="left_%i" type="typeWE" route="left" depart="%i" />' % (
                    vehNr, i), file=routes)
                vehNr += 1
            if random.uniform(0, 1) < pNS:
                print('    <vehicle id="down_%i" type="typeNS" route="down" depart="%i" color="1,0,0"/>' % (
                    vehNr, i), file=routes)
                vehNr += 1
        print("</routes>", file=routes)

# The program looks like this
#    <tlLogic id="0" type="static" programID="0" offset="0">
# the locations of the tls are      NESW
#        <phase duration="31" state="GrGr"/>
#        <phase duration="6"  state="yryr"/>
#        <phase duration="31" state="rGrG"/>
#        <phase duration="6"  state="ryry"/>
#    </tlLogic>


def run():
    """execute the TraCI control loop"""
    step = 0
    # we start with phase 2 where EW has green
    traci.trafficlight.setPhase("0", 2)
    while traci.simulation.getMinExpectedNumber() > 0:
        traci.simulationStep()
        if traci.trafficlight.getPhase("0") == 2:
            # we are not already switching
            if traci.inductionloop.getLastStepVehicleNumber("0") > 0:
                # there is a vehicle from the north, switch
                traci.trafficlight.setPhase("0", 3)
            else:
                # otherwise try to keep green for EW
                traci.trafficlight.setPhase("0", 2)
        step += 1
    traci.close()
    sys.stdout.flush()


def get_options():
    optParser = optparse.OptionParser()
    optParser.add_option("--nogui", action="store_true",
                         default=False, help="run the commandline version of sumo")
    options, args = optParser.parse_args()
    return options


# this is the main entry point of this script
if __name__ == "__main__":
    options = get_options()

    # this script has been called from the command line. It will start sumo as a
    # server, then connect and run
    if options.nogui:
        sumoBinary = checkBinary('sumo')
    else:
        sumoBinary = checkBinary('sumo-gui')

    # first, generate the route file for this simulation
    generate_routefile()

    # this is the normal way of using traci. sumo is started as a
    # subprocess and then the python script connects and runs
    traci.start([sumoBinary, "-c", "data/cross.sumocfg",
                             "--tripinfo-output", "tripinfo.xml"])
    run()



#!/usr/bin/env python
# Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
# Copyright (C) 2011-2022 German Aerospace Center (DLR) and others.
# This program and the accompanying materials are made available under the
# terms of the Eclipse Public License 2.0 which is available at
# https://www.eclipse.org/legal/epl-2.0/
# This Source Code may also be made available under the following Secondary
# Licenses when the conditions for such availability set forth in the Eclipse
# Public License 2.0 are satisfied: GNU General Public License, version 2
# or later which is available at
# https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
# SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later

# @file    runner.py
# @author  Michael Behrisch
# @author  Daniel Krajzewicz
# @date    2011-10-28

from __future__ import absolute_import
from __future__ import print_function

import os
import sys
import subprocess
sys.path.append(os.path.join(os.environ["SUMO_HOME"], "tools"))
import sumolib  # noqa

os.chdir('data')
if "taxi" in sys.argv:
    subprocess.call([sys.executable, "createNetTaxi.py"])
    if "cyber" in sys.argv:
        subprocess.call([sumolib.checkBinary("sumo"), "park15_cyber.sumocfg"])
    else:
        subprocess.call([sumolib.checkBinary("sumo"), "park15.sumocfg"])
else:
    subprocess.call([sys.executable, "createNet.py"])
    options = ["-t"]
    if "cyber" in sys.argv:
        options.append("-c")
    if "agent" in sys.argv:
        subprocess.call([sys.executable, "agentManager.py"] + options)
    else:
        subprocess.call([sys.executable, "simpleManager.py"] + options)



#!/usr/bin/env python
# Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
# Copyright (C) 2008-2022 German Aerospace Center (DLR) and others.
# This program and the accompanying materials are made available under the
# terms of the Eclipse Public License 2.0 which is available at
# https://www.eclipse.org/legal/epl-2.0/
# This Source Code may also be made available under the following Secondary
# Licenses when the conditions for such availability set forth in the Eclipse
# Public License 2.0 are satisfied: GNU General Public License, version 2
# or later which is available at
# https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
# SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later

# @file    doItAll.py
# @author  Michael Behrisch
# @date    2012-02-15

from __future__ import absolute_import
from __future__ import print_function

import sys
import os
import random
sys.path.append(os.path.join(os.environ["SUMO_HOME"], "tools"))
import sumolib  # noqa

netFile = "ttDistro.net.xml"
routeFile = "ttDistro.rou.xml"
configFile = "ttDistro.sumocfg"
dumpFile = "ttDump.xml"


def generateNet(nLanes=2):
    # writing the node file is not needed
    # writing edge file (for different numbers of lanes!!),
    fp = open('ttDistro.edg.xml', 'w')
    rest = 'numLanes="' + repr(nLanes) + '" speed="40" />'
    print('<edges>', file=fp)
    print('\t<edge from="node1" id="1to2" to="node2" ' + rest, file=fp)
    print('\t<edge from="node2" id="2to3" to="node3" ' + rest, file=fp)
    print('\t<edge from="node0" id="0to1" to="node1" ' + rest, file=fp)
    print('</edges>', file=fp)
    fp.close()

    fp = open('ttDistro.nod.xml', 'w')
    print('<nodes>', file=fp)
    print('\t<node id="node0" x="-200.0" y="0.0" />', file=fp)
    print('\t<node id="node1" x="0.0" y="0.0" />', file=fp)
    print('\t<node id="node2" x="+9800.0" y="0.0" />', file=fp)
    print('\t<node id="node3" x="+10000.0" y="0.0" />', file=fp)
    print('</nodes>', file=fp)
    fp.close()
    os.system('%s -n ttDistro.nod.xml -e ttDistro.edg.xml -o %s' % (sumolib.checkBinary("netconvert"),
                                                                    netFile))


def generateDemand(pin, nLanes):
    fp = open(routeFile, 'w')
    fp.write('<routes>\n')
    # normal vehicles
    lFast = 5.0
    lMed = 5.0
    lSlow = 11.0
    vSlow = 25.0
    vMed = 30.0
    vFast = 35.0
    pTruck = 0.1
    pMed = 0.5
    pFast = 0.4

    # fast passenger cars
    s0 = '\t<vType accel="2.0" decel="5.0" id="fast" length="' + \
        repr(lFast) + '" maxSpeed="' + repr(vFast)
    print(s0 + '" sigma="0.8" />', file=fp)

    # slow passenger cars
    s0 = '\t<vType accel="2.0" decel="5.0" id="med" length="' + \
        repr(lMed) + '" maxSpeed="' + repr(vMed)
    print(s0 + '" sigma="0.8" />', file=fp)

    # trucks
    s0 = '\t<vType accel="1.0" decel="4.0" id="truck" length="' + \
        repr(lSlow) + '" maxSpeed="' + repr(vSlow)
    print(s0 + '" sigma="0.6" />', file=fp)

    # the one and only route...
    fp.write('\t<route id="route01" edges="0to1 1to2 2to3"/>\n')

    # now, the vehicles...
    # generate nMax vehicles for each demand ranging from 0.03 to 0.66 veh/s/lane in steps of 0.03 vehs/s/lane
    # use only the last 1000 or so for the statistical analysis
    nMax = 1500
    vehID = 0

    for t in range(100000):
        for lane in range(0, nLanes):
            if random.random() < pin:
                vehID += 1
                s = '\t<vehicle depart="' + \
                    repr(t) + '" arrivalPos="-1" id="' + repr(vehID) + \
                    '" route="route01" departPos="pwagSimple"'
                if (lane == 0):
                    if random.random() < pTruck / (pTruck + 0.5 * pMed):
                        s = s + ' type="truck" departLane="' + \
                            repr(lane) + '" departSpeed="max" />'
                    else:
                        s = s + ' type="med" departLane="' + \
                            repr(lane) + '" departSpeed="max" />'
                else:  # currently: the left lane only in this else cause
                    if random.random() < pFast / (pFast + 0.5 * pMed):
                        s = s + ' type="fast" departLane="' + \
                            repr(lane) + '" departSpeed="max" />'
                    else:
                        s = s + ' type="med" departLane="' + \
                            repr(lane) + '" departSpeed="max" />'
                print(s, file=fp)
        if vehID >= nMax:
            break
    print('</routes>', file=fp)
    fp.close()


def runSim():
    fp = open(configFile, 'w')
    print("""<configuration>
    <input>
        <net-file value="%s"/>
        <route-files value="%s"/>
    </input>
    <report>
        <no-duration-log value="true"/>
        <no-step-log value="true"/>
        <max-depart-delay value="0"/>
        <tripinfo-output value="%s"/>
    </report>
</configuration>""" % (netFile, routeFile, dumpFile), file=fp)
    fp.close()
    os.system('%s -c %s' % (sumolib.checkBinary("sumo"), configFile))


def analyzeData(pp):
    fp = open('gw.txt', 'w')
    n0 = 500
    for line in open(dumpFile):
        ll = line.split('id="')
        if len(ll) > 1:
            vNr = int(ll[1].split('"')[0])
            lll = ll[1].split('duration="')
            tT = float(lll[1].split('"')[0])
            tmp = line.split('depart="')[1]
            t1 = float(tmp.split('"')[0])
            if vNr > n0 and t1 > 0.0:
                print(pp, vNr, t1, tT, file=fp)
    fp.close()


def writeVSSFile():
    # currently not needed.
    # and the vss file,
    fp = open('input_vss.add.xml', 'w')
    print('<additional>', file=fp)
    s = '\t<variableSpeedSign id="vss" lanes="2to3_0'
    maxLanes = None
    for lane in range(1, maxLanes):
        s = s + ' 2to3_' + repr(lane)
    print(s + '" file="laneFlows.vss.xml"/>', file=fp)
    print('</additional>', file=fp)
    fp.close()


# here is "main"
nLanes = 2
print("# preparing the simulation...")
generateNet(nLanes)
for p in range(3, 67, 3):
    generateDemand(0.01 * p, nLanes)
    print("# running the simulation...")
    runSim()
    print("# analyzing the results...")
    analyzeData(p)
    break
print("# All DONE.")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
# Copyright (C) 2008-2022 German Aerospace Center (DLR) and others.
# This program and the accompanying materials are made available under the
# terms of the Eclipse Public License 2.0 which is available at
# https://www.eclipse.org/legal/epl-2.0/
# This Source Code may also be made available under the following Secondary
# Licenses when the conditions for such availability set forth in the Eclipse
# Public License 2.0 are satisfied: GNU General Public License, version 2
# or later which is available at
# https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
# SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later

# @file    runner.py
# @author  Michael Behrisch
# @author  Daniel Krajzewicz
# @date    2011-03-04


from __future__ import print_function
from __future__ import absolute_import
import os
import sys
sys.path.append(os.path.join(
    os.path.dirname(sys.argv[0]), "..", "..", "..", "..", "..", "tools"))
import traci  # noqa
import sumolib  # noqa

traci.start([sumolib.checkBinary('sumo'), "-c", "sumo.sumocfg"] + sys.argv[1:])
for step in range(3):
    print("step", step)
    traci.simulationStep()
print("vehicletypes", traci.vehicletype.getIDList())
print("vehicletype count", traci.vehicletype.getIDCount())
typeID = "DEFAULT_VEHTYPE"
print("examining", typeID)
print("length", traci.vehicletype.getLength(typeID))
print("maxSpeed", traci.vehicletype.getMaxSpeed(typeID))
print("speedFactor", traci.vehicletype.getSpeedFactor(typeID))
print("speedDev", traci.vehicletype.getSpeedDeviation(typeID))
print("accel", traci.vehicletype.getAccel(typeID))
print("decel", traci.vehicletype.getDecel(typeID))
print("emergencyDecel", traci.vehicletype.getEmergencyDecel(typeID))
print("apparentDecel", traci.vehicletype.getApparentDecel(typeID))
print("actionStepLength", traci.vehicletype.getActionStepLength(typeID))
print("imperfection", traci.vehicletype.getImperfection(typeID))
print("tau", traci.vehicletype.getTau(typeID))
print("vClass", traci.vehicletype.getVehicleClass(typeID))
print("emissionclass", traci.vehicletype.getEmissionClass(typeID))
print("shape", traci.vehicletype.getShapeClass(typeID))
print("MinGap", traci.vehicletype.getMinGap(typeID))
print("width", traci.vehicletype.getWidth(typeID))
print("color", traci.vehicletype.getColor(typeID))
print("maxSpeedLat", traci.vehicletype.getMaxSpeedLat(typeID))
print("latAlign", traci.vehicletype.getLateralAlignment(typeID))
print("minGapLat", traci.vehicletype.getMinGapLat(typeID))
print("personCapacity", traci.vehicletype.getPersonCapacity(typeID))
traci.vehicletype.subscribe(typeID)
print(traci.vehicletype.getSubscriptionResults(typeID))
for step in range(3, 6):
    print("step", step)
    traci.simulationStep()
    print(traci.vehicletype.getSubscriptionResults(typeID))
traci.vehicletype.setLength(typeID, 1.0)
print("length", traci.vehicletype.getLength(typeID))
traci.vehicletype.setScale(typeID, 1.4)
print("scale", traci.vehicletype.getScale(typeID))
traci.vehicletype.setMaxSpeed(typeID, 1.0)
print("maxSpeed", traci.vehicletype.getMaxSpeed(typeID))
traci.vehicletype.setSpeedFactor(typeID, 1.1)
print("speedFactor", traci.vehicletype.getSpeedFactor(typeID))
traci.vehicletype.setSpeedDeviation(typeID, 1.1)
print("speedDev", traci.vehicletype.getSpeedDeviation(typeID))
traci.vehicletype.setAccel(typeID, 1.1)
print("accel", traci.vehicletype.getAccel(typeID))
traci.vehicletype.setDecel(typeID, 1.1)
print("decel", traci.vehicletype.getDecel(typeID))
traci.vehicletype.setEmergencyDecel(typeID, 2.2)
print("emergencyDecel", traci.vehicletype.getEmergencyDecel(typeID))
traci.vehicletype.setApparentDecel(typeID, 3.3)
print("apparentDecel", traci.vehicletype.getApparentDecel(typeID))
traci.vehicletype.setActionStepLength(typeID, 2.2)
print("actionStepLength", traci.vehicletype.getActionStepLength(typeID))
traci.vehicletype.setImperfection(typeID, 0.1)
print("imperfection", traci.vehicletype.getImperfection(typeID))
traci.vehicletype.setTau(typeID, 1.1)
print("tau", traci.vehicletype.getTau(typeID))
traci.vehicletype.setVehicleClass(typeID, "bicycle")
print("vClass", traci.vehicletype.getVehicleClass(typeID))
traci.vehicletype.setEmissionClass(typeID, "zero")
print("emissionclass", traci.vehicletype.getEmissionClass(typeID))
traci.vehicletype.setShapeClass(typeID, "bicycle")
print("shape", traci.vehicletype.getShapeClass(typeID))
traci.vehicletype.setMinGap(typeID, 1.1)
print("MinGap", traci.vehicletype.getMinGap(typeID))
traci.vehicletype.setMinGapLat(typeID, 0.6)
print("MinGapLat", traci.vehicletype.getMinGapLat(typeID))
traci.vehicletype.setMaxSpeedLat(typeID, 1.0)
print("MaxSpeedLat", traci.vehicletype.getMaxSpeedLat(typeID))
traci.vehicletype.setLateralAlignment(typeID, "center")
print("LateralAlignment", traci.vehicletype.getLateralAlignment(typeID))
traci.vehicletype.setWidth(typeID, 1.1)
print("width", traci.vehicletype.getWidth(typeID))
traci.vehicletype.setHeight(typeID, 1.9)
print("height", traci.vehicletype.getHeight(typeID))
traci.vehicletype.setColor(typeID, (255, 0, 0, 255))
print("color", traci.vehicletype.getColor(typeID))
copyID = typeID + "_copy"
print("Copying vType '%s' to '%s'" % (typeID, copyID))
traci.vehicletype.copy(typeID, copyID)
print("vehicletypes", traci.vehicletype.getIDList())
print("vehicletype count", traci.vehicletype.getIDCount())
traci.vehicletype.setAccel(copyID, 100.)
print("accel (original)", traci.vehicletype.getAccel(typeID))
print("accel (copied)", traci.vehicletype.getAccel(copyID))

traci.simulationStep()
traci.close()



#!/usr/bin/env python
# Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
# Copyright (C) 2008-2022 German Aerospace Center (DLR) and others.
# This program and the accompanying materials are made available under the
# terms of the Eclipse Public License 2.0 which is available at
# https://www.eclipse.org/legal/epl-2.0/
# This Source Code may also be made available under the following Secondary
# Licenses when the conditions for such availability set forth in the Eclipse
# Public License 2.0 are satisfied: GNU General Public License, version 2
# or later which is available at
# https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
# SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later

# @file    runner.py
# @author  Daniel Krajzewicz
# @author  Michael Behrisch
# @date    2010-03-02

from __future__ import absolute_import
from __future__ import print_function

import os
import subprocess
import sys
sys.path.append(
    os.path.join(os.path.dirname(sys.argv[0]), '..', '..', '..', '..', "tools"))
import sumolib  # noqa

netconvertBinary = sumolib.checkBinary('netconvert')
duarouterBinary = sumolib.checkBinary('duarouter')
sumoBinary = sumolib.checkBinary('sumo')

print(">>> Building the xml network")
sys.stdout.flush()
subprocess.call([netconvertBinary, "-c", "netconvert.netccfg"],
                stdout=sys.stdout, stderr=sys.stderr)
print(">>> Building the binary network")
sys.stdout.flush()
subprocess.call([netconvertBinary, "-c", "netconvert.netccfg",
                 "-o", "circular.net.xml.gz"], stdout=sys.stdout, stderr=sys.stderr)
sys.stdout.flush()
print(">>> Converting the routes to binary")
subprocess.call([duarouterBinary, "-c", "duarouter.duarcfg"],
                stdout=sys.stdout, stderr=sys.stderr)
sys.stdout.flush()
print(">>> Running Simulation with binary input")
subprocess.call(
    [sumoBinary, "-c", "sumo.sumocfg"], stdout=sys.stdout, stderr=sys.stderr)



# -*- coding: utf-8 -*-
# Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
# Copyright (C) 2008-2022 German Aerospace Center (DLR) and others.
# This program and the accompanying materials are made available under the
# terms of the Eclipse Public License 2.0 which is available at
# https://www.eclipse.org/legal/epl-2.0/
# This Source Code may also be made available under the following Secondary
# Licenses when the conditions for such availability set forth in the Eclipse
# Public License 2.0 are satisfied: GNU General Public License, version 2
# or later which is available at
# https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
# SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later

# @file    constants.py
# @author  Michael Behrisch
# @author  Daniel Krajzewicz
# @date    2008-07-21

from __future__ import absolute_import
import os
import sys

INFINITY = 1e400

PREFIX = "park"
DOUBLE_ROWS = 8
ROW_DIST = 29
STOP_POS = ROW_DIST - 9
SLOTS_PER_ROW = 10
SLOT_WIDTH = 5
SLOT_LENGTH = 9
SLOT_FOOT_LENGTH = 5
CAR_CAPACITY = 3
CYBER_CAPACITY = 20
BUS_CAPACITY = 30
TOTAL_CAPACITY = 60
CYBER_SPEED = 5
CYBER_LENGTH = 9
WAIT_PER_PERSON = 5
OCCUPATION_PROBABILITY = 0.5
BREAK_DELAY = 1200

SUMO_HOME = os.path.realpath(os.environ.get(
    "SUMO_HOME", os.path.join(os.path.dirname(__file__), "..", "..", "..", "..")))
sys.path.append(os.path.join(SUMO_HOME, "tools"))
try:
    from sumolib import checkBinary  # noqa
except ImportError:
    def checkBinary(name):
        return name
NETCONVERT = checkBinary("netconvert")
SUMO = checkBinary("sumo")
SUMOGUI = checkBinary("sumo-gui")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
# Copyright (C) 2008-2022 German Aerospace Center (DLR) and others.
# This program and the accompanying materials are made available under the
# terms of the Eclipse Public License 2.0 which is available at
# https://www.eclipse.org/legal/epl-2.0/
# This Source Code may also be made available under the following Secondary
# Licenses when the conditions for such availability set forth in the Eclipse
# Public License 2.0 are satisfied: GNU General Public License, version 2
# or later which is available at
# https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
# SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later

# @file    runner.py
# @author  Jakob Erdmann
# @author  Laura Bieker
# @author  Michael Behrisch
# @date    2011-05-27

"""
import osm network
then import the built network again and check for idempotency
"""
from __future__ import absolute_import

import sys
import os
import subprocess
import difflib
if "SUMO_HOME" in os.environ:
    sys.path.append(os.path.join(os.environ["SUMO_HOME"], "tools"))
import sumolib  # noqa

osm_input = 'osm.xml'
net_output = 'from_osm'
net_output2 = 'net'

netconvert = sumolib.checkBinary('netconvert')

# filter header and projection clause


def filtered(file_name, start_element):
    skippedHeader = False
    with open(file_name) as f:
        for line in f:
            if start_element in line:
                skippedHeader = True
            if not skippedHeader:
                continue
            if '<location' in line:
                continue
            if '<projection' in line:
                continue
            yield line


def get_filtered_lines(prefix):
    result = []
    for suffix, start_element in [
            ('.net.xml', '<net '),
            ('.nod.xml', '<nodes '),
            ('.edg.xml', '<edges '),
            ('.con.xml', '<connections '),
            ('.tll.xml', '<tlLogics ')]:
        result += list(filtered(prefix + suffix, start_element))
    return result


args1 = [netconvert,
         '--no-internal-links',
         '--osm-files',  osm_input, '--proj.utm',
         '-R', '--ramps.guess',
         '--tls.guess', '--tls.join',
         '--junctions.join',
         '--plain-output-prefix', net_output,
         '--output', net_output + '.net.xml']

args2 = [netconvert,
         '--sumo-net-file', net_output + '.net.xml',
         '--no-internal-links',
         '--offset.disable-normalization',
         '--plain-output-prefix', net_output2,
         '--output', net_output2 + '.net.xml']

subprocess.call(args1)
subprocess.call(args2)

fromlines = get_filtered_lines(net_output)
tolines = get_filtered_lines(net_output2)
# with open('fromlines','w') as f: f.write('\n'.join(fromlines))
# with open('tolines','w') as f: f.write('\n'.join(tolines))
sys.stderr.writelines(difflib.unified_diff(fromlines, sumolib.fpdiff.fpfilter(fromlines, tolines, 0.0201)))



# -*- coding: utf-8 -*-
# Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
# Copyright (C) 2008-2022 German Aerospace Center (DLR) and others.
# This program and the accompanying materials are made available under the
# terms of the Eclipse Public License 2.0 which is available at
# https://www.eclipse.org/legal/epl-2.0/
# This Source Code may also be made available under the following Secondary
# Licenses when the conditions for such availability set forth in the Eclipse
# Public License 2.0 are satisfied: GNU General Public License, version 2
# or later which is available at
# https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
# SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later

# @file    vehicleControl.py
# @author  Michael Behrisch
# @author  Daniel Krajzewicz
# @author  Lena Kalleske
# @date    2008-07-21

from __future__ import absolute_import
from __future__ import print_function
import random
import sys
import os
from optparse import OptionParser

from constants import PREFIX, DOUBLE_ROWS, STOP_POS, SLOTS_PER_ROW, SLOT_LENGTH, BUS_CAPACITY, BREAK_DELAY
from constants import CYBER_CAPACITY
import statistics

if 'SUMO_HOME' in os.environ:
    tools = os.path.join(os.environ['SUMO_HOME'], 'tools')
    sys.path.append(tools)
else:
    sys.exit("please declare environment variable 'SUMO_HOME'")

import traci  # noqa
import traci.constants as tc  # noqa
import sumolib  # noqa


class Manager:

    def personArrived(self, vehicleID, edge, target):
        raise NotImplementedError

    def cyberCarArrived(self, vehicleID, edge):
        raise NotImplementedError

    def cyberCarBroken(self, vehicleID, edge):
        pass

    def setNewTargets(self):
        pass


class Status:

    def __init__(self, edge, pos):
        self.edge = edge
        self.pos = pos
        self.parking = False
        self.target = None
        self.targetPos = None
        self.slot = None
        self.delay = None

    def __repr__(self):
        return "%s,%s" % (self.edge, self.pos)


class Setting:
    step = 0
    manager = None
    verbose = False
    cyber = False
    breakstep = None


setting = Setting()
occupancy = {}
vehicleStatus = {}
persons = {}
waiting = {}


def init(manager):
    optParser = OptionParser()
    optParser.add_option("-v", "--verbose", action="store_true",
                         default=False, help="tell me what you are doing")
    optParser.add_option("-g", "--gui", action="store_true",
                         default=False, help="run with GUI")
    optParser.add_option("-c", "--cyber", action="store_true",
                         default=False, help="use small cybercars instead of big busses")
    optParser.add_option("-d", "--demand", type="int",
                         default=15, help="period with which the persons are emitted")
    optParser.add_option("-b", "--break", type="int", dest="breakstep", metavar="TIMESTEP",
                         help="let a vehicle break for %s seconds at TIMESTEP" % BREAK_DELAY)
    optParser.add_option("-t", "--test", action="store_true",
                         default=False, help="Run in test mode")
    options, _ = optParser.parse_args()
    sumoExe = sumolib.checkBinary('sumo')
    if options.gui:
        sumoExe = sumolib.checkBinary('sumo-gui')
    sumoConfig = "%s%02i.sumocfg" % (PREFIX, options.demand)
    if options.cyber:
        sumoConfig = "%s%02i_cyber.sumocfg" % (PREFIX, options.demand)
    traci.start([sumoExe, "-c", sumoConfig])
    traci.simulation.subscribe()
    setting.manager = manager
    setting.verbose = options.verbose
    setting.cyber = options.cyber
    setting.breakstep = options.breakstep
    try:
        while setting.step < 100 or statistics.personsRunning > 0:
            doStep()
        statistics.evaluate(options.test)
    finally:
        traci.close()


def getCapacity():
    if setting.cyber:
        return CYBER_CAPACITY
    return BUS_CAPACITY


def getStep():
    return setting.step


def getPosition(vehicleID):
    return vehicleStatus[vehicleID].edge


def stopAt(vehicleID, edge, pos=None):
    if pos is None:
        pos = STOP_POS
        if edge.endswith("out") or edge.endswith("in"):
            pos = 90.
    traci.vehicle.changeTarget(vehicleID, edge)
    if setting.verbose:
        print("stopAt", vehicleID, edge, pos)
#        print vehicleStatus[vehicleID]
#        print traci.vehicle.getRoute(vehicleID)
    traci.vehicle.setStop(vehicleID, edge, pos)
    vehicleStatus[vehicleID].target = edge
    vehicleStatus[vehicleID].targetPos = pos


def leaveStop(vehicleID, newTarget=None, delay=0):
    v = vehicleStatus[vehicleID]
    if newTarget:
        traci.vehicle.changeTarget(vehicleID, newTarget)
    traci.vehicle.setStop(vehicleID, v.target, v.targetPos, 0, delay)
    v.target = None
    v.targetPos = None
    v.parking = False


def _rerouteCar(vehicleID):
    slotEdge = ""
    for rowIdx in range(DOUBLE_ROWS):
        for idx in range(SLOTS_PER_ROW):
            for dir in ["l", "r"]:
                slotEdge = "slot%s-%s%s" % (rowIdx, idx, dir)
                if slotEdge not in occupancy:
                    occupancy[slotEdge] = vehicleID
                    stopAt(vehicleID, slotEdge, SLOT_LENGTH - 5.)
                    return


def _reroutePersons(edge):
    if edge in persons:
        for person in persons[edge]:
            if not vehicleStatus[person].slot:
                row = int(edge[4])
                targetEdge = "footmain%sto%s" % (row, row + 1)
                traci.vehicle.setStop(
                    person, edge.replace("slot", "-foot"), 1., 0, 0)
                stopAt(person, targetEdge)
                vehicleStatus[person].parking = False
                vehicleStatus[person].slot = edge


def _checkInitialPositions(vehicleID, edge, pos):
    if vehicleID in vehicleStatus:
        vehicleStatus[vehicleID].edge = edge
        vehicleStatus[vehicleID].pos = pos
    else:
        vehicleStatus[vehicleID] = Status(edge, pos)
        if edge == "mainin":
            _rerouteCar(vehicleID)
        elif edge == "cyberin":
            stopAt(vehicleID, "cyberin")
        elif edge == "footfairin":
            stopAt(vehicleID, "footmainout")
        elif "foot" in edge:
            traci.vehicle.setStop(vehicleID, "-" + edge)
            parkEdge = edge.replace("foot", "slot")
            if parkEdge not in persons:
                persons[parkEdge] = []
            persons[parkEdge].append(vehicleID)
            vehicleStatus[vehicleID].parking = True
        elif edge.startswith("slot"):
            stopAt(vehicleID, edge, SLOT_LENGTH - 5.)
            occupancy[edge] = vehicleID
    _reroutePersons(edge)


def doStep():
    setting.step += 1
    if setting.verbose:
        print("step", setting.step)
    traci.simulationStep()
    moveNodes = []
    for veh, subs in traci.vehicle.getAllSubscriptionResults().items():
        moveNodes.append((veh, subs[tc.VAR_ROAD_ID], subs[tc.VAR_LANEPOSITION]))
    departed = traci.simulation.getSubscriptionResults()[tc.VAR_DEPARTED_VEHICLES_IDS]
    for v in departed:
        traci.vehicle.subscribe(v)
        subs = traci.vehicle.getSubscriptionResults(v)
        moveNodes.append((v, subs[tc.VAR_ROAD_ID], subs[tc.VAR_LANEPOSITION]))
    for vehicleID, edge, pos in moveNodes:
        _checkInitialPositions(vehicleID, edge, pos)
        vehicle = vehicleStatus[vehicleID]
        if edge == vehicle.target and not vehicle.parking:
            if edge.startswith("footmain"):
                vehicle.parking = True
                target = "footmainout"
                if edge == "footmainout":
                    row = random.randrange(0, DOUBLE_ROWS)
                    target = "footmain%sto%s" % (row, row + 1)
                statistics.personArrived(vehicleID, edge, target, setting.step)
                setting.manager.personArrived(vehicleID, edge, target)
            if edge.startswith("cyber"):
                if vehicle.delay:
                    vehicle.delay -= 1
                elif setting.breakstep and setting.step >= setting.breakstep and edge != "cyberin":
                    if setting.verbose:
                        print("broken", vehicleID, edge)
                    setting.breakstep = None
                    setting.manager.cyberCarBroken(vehicleID, edge)
                    vehicle.delay = BREAK_DELAY
                else:
                    if setting.verbose:
                        print("arrived", vehicleID, edge)
                    vehicle.parking = True
                    setting.manager.cyberCarArrived(vehicleID, edge)
    setting.manager.setNewTargets()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
# Copyright (C) 2008-2022 German Aerospace Center (DLR) and others.
# This program and the accompanying materials are made available under the
# terms of the Eclipse Public License 2.0 which is available at
# https://www.eclipse.org/legal/epl-2.0/
# This Source Code may also be made available under the following Secondary
# Licenses when the conditions for such availability set forth in the Eclipse
# Public License 2.0 are satisfied: GNU General Public License, version 2
# or later which is available at
# https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
# SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later

# @file    runner.py
# @author  Jakob Erdmann
# @author  Michael Behrisch
# @date    2013-08-16

"""
import sumo network then export to the given format and import again
and check for idempotency with the original sumo network
"""
from __future__ import absolute_import


import sys
import os
import subprocess
sys.path.append(
    os.path.join(os.path.dirname(__file__), '..', '..', '..', "tools"))
import sumolib  # noqa

format = sys.argv[1]
netconvert = sumolib.checkBinary('netconvert')

args1 = [netconvert,
         '--sumo-net-file', 'input_net.net.xml',
         '--%s-output' % format, format]

args2 = [netconvert, '--%s' % format, format] + sys.argv[2:]
if format == "matsim":
    args2 += ["--xml-validation", "never"]

subprocess.call(args1)
subprocess.call(args2)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
# Copyright (C) 2008-2022 German Aerospace Center (DLR) and others.
# This program and the accompanying materials are made available under the
# terms of the Eclipse Public License 2.0 which is available at
# https://www.eclipse.org/legal/epl-2.0/
# This Source Code may also be made available under the following Secondary
# Licenses when the conditions for such availability set forth in the Eclipse
# Public License 2.0 are satisfied: GNU General Public License, version 2
# or later which is available at
# https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
# SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later

# @file    runner.py
# @author  Jakob Erdmann
# @date    2017-01-23


from __future__ import print_function
from __future__ import absolute_import
import os
import sys

if "SUMO_HOME" in os.environ:
    sys.path.append(os.path.join(os.environ["SUMO_HOME"], "tools"))

import traci  # noqa
import sumolib  # noqa

sumoBinary = sumolib.checkBinary("sumo")
cmd = [sumoBinary,
       '-n', 'input_net2.net.xml',
       '--no-step-log',
       # '-S', '-Q',
       ]

ANGLE_UNDEF = traci.constants.INVALID_DOUBLE_VALUE
INVALID = traci.constants.INVALID_DOUBLE_VALUE

vehID = "v0"


def check(x, y, angle, exLane, exPos, exPosLat, comment, edgeHint="",
          laneHint=0, keepRoute=1):
    traci.vehicle.moveToXY(vehID, edgeHint, laneHint, x, y, angle, keepRoute)
    traci.simulationStep()
    x2, y2 = traci.vehicle.getPosition(vehID)
    lane2 = traci.vehicle.getLaneID(vehID)
    pos2 = traci.vehicle.getLanePosition(vehID)
    posLat2 = traci.vehicle.getLateralLanePosition(vehID)
    if (abs(x - x2) > 0.1 or
            abs(y - y2) > 0.1 or
            (exLane != lane2 and exLane is not None) or
            (exPos is not None and abs(exPos - pos2) > 0.1) or
            (exPosLat is not None and abs(exPosLat - posLat2) > 0.1)):
        print(comment, ("failed: x=%s, x2=%s,   y=%s, y2=%s,   lane=%s, lane2=%s, pos=%s, pos2=%s   " +
                        "posLat=%s posLat2=%s") % (x, x2, y, y2, exLane, lane2, exPos, pos2, exPosLat, posLat2))
    else:
        # (comment, "success")
        pass
    print(traci.simulation.getTime(),
          " lane=%s" % lane2,
          # " route=%s" % str(traci.vehicle.getRoute(vehID)),
          " right=%s, %s" % traci.vehicle.getLaneChangeStatePretty(vehID, -1),
          " left=%s, %s" % traci.vehicle.getLaneChangeStatePretty(vehID,  1),
          " accel=%s" % traci.vehicle.getAcceleration(vehID),
          " nextTLS=%s" % traci.vehicle.getNextTLS(vehID),
          " leader=%s" % str(traci.vehicle.getLeader(vehID, 500)),
          )


traci.start(cmd)
traci.setLegacyGetLeader(False)
traci.simulationStep()
traci.route.add("SE", ["SC", "CE"])
traci.vehicle.add(vehID, "SE")
check(104.95, 20, ANGLE_UNDEF, None, None, None, "correct lane")
check(104.95, 30, ANGLE_UNDEF, None, None, None, "correct lane")
check(104.95, 40, ANGLE_UNDEF, None, None, None, "correct lane")
check(101.65, 50, ANGLE_UNDEF, None, None, None, "correct lane")
check(101.65, 60, ANGLE_UNDEF, None, None, None, "correct lane")
check(101.65, 70, ANGLE_UNDEF, None, None, None, "correct lane")
check(101.65, 80, ANGLE_UNDEF, None, None, None, "correct lane")
# test misleading lane hints (vehicle is on SC_1 but position is on SC_0
check(104.95, 90, ANGLE_UNDEF, None, None, None, "inconsistent hint", edgeHint="SC", laneHint=1)
check(104.95, 20, ANGLE_UNDEF, None, None, None, "inconsistent hint", edgeHint="SC", laneHint=1, keepRoute=2)
traci.close()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
# Copyright (C) 2008-2022 German Aerospace Center (DLR) and others.
# This program and the accompanying materials are made available under the
# terms of the Eclipse Public License 2.0 which is available at
# https://www.eclipse.org/legal/epl-2.0/
# This Source Code may also be made available under the following Secondary
# Licenses when the conditions for such availability set forth in the Eclipse
# Public License 2.0 are satisfied: GNU General Public License, version 2
# or later which is available at
# https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
# SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later

# @file    runner.py
# @author  Michael Behrisch
# @author  Daniel Krajzewicz
# @date    2011-03-04

from __future__ import print_function
from __future__ import absolute_import
import os
import sys

if "SUMO_HOME" in os.environ:
    sys.path.append(os.path.join(os.environ["SUMO_HOME"], "tools"))
import traci  # noqa
import sumolib  # noqa

traci.start([sumolib.checkBinary('sumo'), "-n", "input_net3.net.xml",
             "--no-step-log"] + sys.argv[1:])
for step in range(3):
    print("step", step)
    traci.simulationStep()
laneID = ":C_15_0"
print("examining", laneID)
print("length", traci.lane.getLength(laneID))
print("maxSpeed", traci.lane.getMaxSpeed(laneID))
print("width", traci.lane.getWidth(laneID))
print("allowed", traci.lane.getAllowed(laneID))
print("disallowed", traci.lane.getDisallowed(laneID))
print("linkNum", traci.lane.getLinkNumber(laneID))
if traci.isLibsumo() or traci.isLibtraci():
    print("links", [link[:4] for link in traci.lane.getLinks(laneID)])
    print("linksExtended", traci.lane.getLinks(laneID))
else:
    print("links", traci.lane.getLinks(laneID, extended=False))
    print("linksExtended", traci.lane.getLinks(laneID, extended=True))
print("shape", traci.lane.getShape(laneID))
print("edge", traci.lane.getEdgeID(laneID))
print("CO2", traci.lane.getCO2Emission(laneID))
print("CO", traci.lane.getCOEmission(laneID))
print("HC", traci.lane.getHCEmission(laneID))
print("PMx", traci.lane.getPMxEmission(laneID))
print("NOx", traci.lane.getNOxEmission(laneID))
print("Fuel", traci.lane.getFuelConsumption(laneID))
print("Noise", traci.lane.getNoiseEmission(laneID))
print("Elec", traci.lane.getElectricityConsumption(laneID))
print("meanSpeed", traci.lane.getLastStepMeanSpeed(laneID))
print("occupancy", traci.lane.getLastStepOccupancy(laneID))
print("lastLength", traci.lane.getLastStepLength(laneID))
print("traveltime", traci.lane.getTraveltime(laneID))
print("numVeh", traci.lane.getLastStepVehicleNumber(laneID))
print("haltVeh", traci.lane.getLastStepHaltingNumber(laneID))
print("vehIds", traci.lane.getLastStepVehicleIDs(laneID))
print("waiting time", traci.lane.getWaitingTime(laneID))

traci.lane.setAllowed(laneID, ["taxi"])
print("after setAllowed", traci.lane.getAllowed(laneID), traci.lane.getDisallowed(laneID))
traci.lane.setDisallowed(laneID, ["bus"])
print("after setDisallowed", traci.lane.getAllowed(laneID), traci.lane.getDisallowed(laneID))
traci.lane.setMaxSpeed(laneID, 42.)
print("after setMaxSpeed", traci.lane.getMaxSpeed(laneID))
traci.lane.setLength(laneID, 123.)
print("after setLength", traci.lane.getLength(laneID))

traci.lane.subscribe(laneID)
print(traci.lane.getSubscriptionResults(laneID))
for step in range(3, 6):
    print("step", step)
    print("pending", laneID, traci.lane.getPendingVehicles(laneID))
    traci.simulationStep()
    print(traci.lane.getSubscriptionResults(laneID))
traci.close()



#!/usr/bin/env python
# Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
# Copyright (C) 2008-2022 German Aerospace Center (DLR) and others.
# This program and the accompanying materials are made available under the
# terms of the Eclipse Public License 2.0 which is available at
# https://www.eclipse.org/legal/epl-2.0/
# This Source Code may also be made available under the following Secondary
# Licenses when the conditions for such availability set forth in the Eclipse
# Public License 2.0 are satisfied: GNU General Public License, version 2
# or later which is available at
# https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
# SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later

# @file    runner.py
# @author  Robert Hilbrich
# @date    2019-11-10

from __future__ import absolute_import
from __future__ import print_function

import os
import subprocess
import sys
toolDir = os.path.join(
    os.path.dirname(__file__), '..', '..', '..', '..', "tools")
sys.path.append(toolDir)
import sumolib  # noqa

sumoBinary = sumolib.checkBinary('sumo')

# Set the example environment variable
os.environ["NETFILENAME"] = "input_net"
os.environ["HOME"] = os.path.abspath(os.curdir)

# write config
subprocess.call([sumoBinary, "-c", "sumo.sumocfg", "--no-step-log", "-C", "config.sumocfg",
                 "--collision-output", "~/collision.xml",
                 "--tripinfo", "${PID}.trips.xml", "--summary", "sum${UTC}.xml", "--log", "log${LOCALTIME}.log"])

# file output direct
subprocess.call([sumoBinary, "-c", "config.sumocfg"])

files = list(sorted(os.listdir()))
print("uncheck:", files, os.curdir)
assert(files[0].endswith(".trips.xml"))
assert(int(files[0][:-10]) > 0)
assert("collision.xml" in files)
assert(len(files) == 11)



#!/usr/bin/env python
# Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
# Copyright (C) 2012-2022 German Aerospace Center (DLR) and others.
# This program and the accompanying materials are made available under the
# terms of the Eclipse Public License 2.0 which is available at
# https://www.eclipse.org/legal/epl-2.0/
# This Source Code may also be made available under the following Secondary
# Licenses when the conditions for such availability set forth in the Eclipse
# Public License 2.0 are satisfied: GNU General Public License, version 2
# or later which is available at
# https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
# SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later

# @file    runner.py
# @author  Daniel Krajzewicz
# @date    2013-10-19


from __future__ import absolute_import
from __future__ import print_function

import subprocess
import sys
import sumodump
import sumotripinfos
import sumodetectors


def call(command, log):
    print("-" * 79, file=log)
    print(command, file=log)
    log.flush()
    print(command)
    retCode = subprocess.call(command, stdout=sys.stdout, stderr=sys.stderr)
    if retCode != 0:
        print("Execution of %s failed. Look into %s for details." % (
            command, log.name), file=sys.stderr)
        sys.exit(retCode)


def iterateGreenTimes(emissionClass, outputFile, T, MIN_GREEN, log):
    fd = open("input_routes.rou.xml", "w")
    print('<routes>', file=fd)
    print('    <!-- vType id="t1" accel="0.8" decel="4.5" sigma="0.5" length="7.5" maxSpeed="36"/ -->	', file=fd)
    print('    <route id="r1" multi_ref="x" edges="e2 e w w2"/>', file=fd)
    for i in range(0, 1000):
        print('    <vehicle id="v.%s" depart="%s" departspeed="13.89" departpos="0" route="r1" type="t1"/>' % (
            i, i * 5), file=fd)
    print('</routes>', file=fd)
    fd.close()

    fdo = open(outputFile, "w")
    for i in range(MIN_GREEN, T - MIN_GREEN):
        print(">>> Building plan (at t=%s)" % (i))
        fd = open("input_additional.add.xml", "w")
        print('<additional>', file=fd)
        print('    <vType id="t1" accel="0.8" decel="4.5" sigma="0.5" length="7.5" maxSpeed="36" emissionClass="%s"/>' %
              emissionClass, file=fd)
        print('    <tlLogic id="c" type="static" programID="my" offset="0.00">', file=fd)
        print('        <phase duration="%s" state="GGrr"/>' % (i), file=fd)
        print('        <phase duration="%s" state="rrGG"/>' % (T - i), file=fd)
        print('    </tlLogic>', file=fd)
        print('    <laneData id="traffic" freq="100000" file="aggregated_traffic.xml"/>', file=fd)
        print('    <laneData id="noise" freq="100000" type="harmonoise" file="aggregated_noise.xml"/>', file=fd)
        print('    <laneData id="emissions" freq="100000" type="hbefa" file="aggregated_emissions.xml"/>', file=fd)
        print('    <e2Detector id="e2" lane="e_0" pos=".1" length="9999.8" freq="100000" file="e2.xml" ' +
              'friendlyPos="x"/>', file=fd)
        print('    <e2Detector id="e2_tls" tl="c" to="w_0" lane="e_0" pos=".1" length="9999.8" freq="100000" ' +
              'file="e2_tls.xml" friendlyPos="x"/>', file=fd)
        print('</additional>', file=fd)
        fd.close()

        print(">>> Simulating (at t=%s)" % (i))
        call(sumoBinary + " -c sumo.sumo.cfg -v", log)

#    shutil.copy("aggregated_traffic.xml", "backup/" + str(i) + "_aggregated_traffic.xml")
#    shutil.copy("aggregated_noise.xml", "backup/" + str(i) + "_aggregated_noise.xml")
#    shutil.copy("aggregated_emissions.xml", "backup/" + str(i) + "_aggregated_emissions.xml")
#    shutil.copy("tripinfos.xml", "backup/" + str(i) + "_tripinfos.xml")
#    shutil.copy("e2.xml", "backup/" + str(i) + "_e2.xml")
#    shutil.copy("e2_tls.xml", "backup/" + str(i) + "_e2_tls.xml")
#    shutil.copy("netstate.xml", "backup/" + str(i) + "_netstate.xml")

        dumpEmissions = sumodump.readDump("aggregated_emissions.xml", ["CO_perVeh", "CO2_perVeh", "HC_perVeh",
                                                                       "PMx_perVeh", "NOx_perVeh", "fuel_perVeh"])
        dumpNoise = sumodump.readDump("aggregated_noise.xml", ["noise"])
        dumpTraffic = sumodump.readDump(
            "aggregated_traffic.xml", ["waitingTime", "speed"])
        tripinfos = sumotripinfos.readTripinfos("tripinfos.xml", ["waitSteps"])
        e2 = sumodetectors.readAreal("e2.xml", ["maxJamLengthInVehicles", "jamLengthInVehiclesSum",
                                                "meanHaltingDuration", "startedHalts"])

        vals = tripinfos.get("waitSteps")
        meanV = 0
        for v in vals:
            meanV += vals[v]
        meanV = meanV / len(vals)

        wtv = dumpTraffic.get("waitingTime")[-1]["e_0"]
        nv = dumpNoise.get("noise")[-1]["e_0"]
        cov = dumpEmissions.get("CO_perVeh")[-1]["e_0"] / 1000.
        co2v = dumpEmissions.get("CO2_perVeh")[-1]["e_0"] / 1000.
        hcv = dumpEmissions.get("HC_perVeh")[-1]["e_0"] / 1000.
        pmv = dumpEmissions.get("PMx_perVeh")[-1]["e_0"] / 1000.
        nov = dumpEmissions.get("NOx_perVeh")[-1]["e_0"] / 1000.
        fv = dumpEmissions.get("fuel_perVeh")[-1]["e_0"] / 1000.
        msv = dumpTraffic.get("speed")[-1]["e_0"]

        mjlv = e2.get("maxJamLengthInVehicles")[-1]["e2"]
        jlsv = e2.get("jamLengthInVehiclesSum")[-1]["e2"]
        mhdv = e2.get("meanHaltingDuration")[-1]["e2"]
        shv = e2.get("startedHalts")[-1]["e2"]

        print("%s;%s;%s;%s;%s;%s;%s;%s;%s;%s;%s;%s;%s;%s;%s;%s" % (
            T - i, wtv, nv, cov, co2v, hcv, pmv, nov, fv, meanV, msv,
            10000. / msv - 10000. / 13.82, mjlv, jlsv, mhdv, shv), file=fdo)
    fdo.close()


MIN_GREEN = 10
T = 180
log = open("logfile.txt", "w")

# os.environ.get("NETCONVERT_BINARY", os.path.join(os.path.dirname(sys.argv[0]), '..', '..', 'bin', 'netconvert'))
netconvertBinary = ".\\netconvert.exe"
call(netconvertBinary + " -c netconvert.netc.cfg -v", log)
sumoBinary = ".\\sumo.exe"

emissionClasses = []
fd = open("all.txt")
for line in fd.readlines():
    line = line.strip()
    if len(line) < 1:
        continue
    emissionClasses.append(line)

for e in emissionClasses:
    of = e + "_results.csv"
    iterateGreenTimes(e, of, T, MIN_GREEN, log)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
# Copyright (C) 2012-2022 German Aerospace Center (DLR) and others.
# This program and the accompanying materials are made available under the
# terms of the Eclipse Public License 2.0 which is available at
# https://www.eclipse.org/legal/epl-2.0/
# This Source Code may also be made available under the following Secondary
# Licenses when the conditions for such availability set forth in the Eclipse
# Public License 2.0 are satisfied: GNU General Public License, version 2
# or later which is available at
# https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
# SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later

# @file    runner.py
# @author  Daniel Krajzewicz
# @date    2013-10-30

# import osm network

import sys
import os

if "SUMO_HOME" in os.environ:
    sys.path.append(os.path.join(os.environ["SUMO_HOME"], "tools"))

import sumolib.net.generator.cross as generator  # noqa
from sumolib.net.generator.network import Edge  # noqa

for i in range(1, 5):
    defaultEdge = Edge(numLanes=i, maxSpeed=13.89)
    net = generator.cross(None, defaultEdge)
    net.build("net_with_%s_lanes.net.xml" % i)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
# Copyright (C) 2008-2022 German Aerospace Center (DLR) and others.
# This program and the accompanying materials are made available under the
# terms of the Eclipse Public License 2.0 which is available at
# https://www.eclipse.org/legal/epl-2.0/
# This Source Code may also be made available under the following Secondary
# Licenses when the conditions for such availability set forth in the Eclipse
# Public License 2.0 are satisfied: GNU General Public License, version 2
# or later which is available at
# https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
# SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later

# @file    runner.py
# @author  Jakob Erdmann
# @date    2017-01-23


from __future__ import print_function
from __future__ import absolute_import
import os
import sys

if "SUMO_HOME" in os.environ:
    sys.path.append(os.path.join(os.environ["SUMO_HOME"], "tools"))

import traci  # noqa
import sumolib  # noqa

ANGLE_UNDEF = traci.constants.INVALID_DOUBLE_VALUE
INVALID = traci.constants.INVALID_DOUBLE_VALUE

vehID = "v0"


def check(x, y, angle, exLane, exPos, exPosLat, comment):
    traci.vehicle.moveToXY(vehID, "", 0, x, y, angle, keepRoute=2)
    traci.simulationStep()
    x2, y2 = traci.vehicle.getPosition(vehID)
    lane2 = traci.vehicle.getLaneID(vehID)
    pos2 = traci.vehicle.getLanePosition(vehID)
    posLat2 = traci.vehicle.getLateralLanePosition(vehID)
    if (abs(x - x2) > 0.1 or
            abs(y - y2) > 0.1 or
            exLane != lane2 or
            (exPos is not None and abs(exPos - pos2) > 0.1) or
            (exPosLat is not None and abs(exPosLat - posLat2) > 0.1)):
        print(comment, ("failed: x=%s, x2=%s,   y=%s, y2=%s,   lane=%s, lane2=%s, pos=%s, pos2=%s   " +
                        "posLat=%s posLat2=%s") % (x, x2, y, y2, exLane, lane2, exPos, pos2, exPosLat, posLat2))
    else:
        # (comment, "success")
        pass


traci.start([sumolib.checkBinary("sumo"), '-n', 'input_net.net.xml', '--no-step-log'])
traci.simulationStep()
traci.route.add("beg", ["beg"])
traci.vehicle.add(vehID, "beg")
traci.simulationStep()
x, y = traci.vehicle.getPosition(vehID)

for i in range(10):
    traci.vehicle.moveToXY(vehID, "", -1, x, y)
    traci.simulationStep()
    t = traci.simulation.getTime()
    x2, y2 = traci.vehicle.getPosition(vehID)
    print("%s x=%s v=%s vOrig=%s" % (t, x2,
                                     traci.vehicle.getSpeed(vehID),
                                     traci.vehicle.getSpeedWithoutTraCI(vehID)))
    x += i

traci.close()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
# Copyright (C) 2008-2022 German Aerospace Center (DLR) and others.
# This program and the accompanying materials are made available under the
# terms of the Eclipse Public License 2.0 which is available at
# https://www.eclipse.org/legal/epl-2.0/
# This Source Code may also be made available under the following Secondary
# Licenses when the conditions for such availability set forth in the Eclipse
# Public License 2.0 are satisfied: GNU General Public License, version 2
# or later which is available at
# https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
# SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later

# @file    runner.py
# @author  Jakob Erdmann
# @date    2015-02-06


from __future__ import print_function
from __future__ import absolute_import
import os
import sys

if "SUMO_HOME" in os.environ:
    sys.path.append(os.path.join(os.environ["SUMO_HOME"], "tools"))
import traci  # noqa
import sumolib  # noqa


def step():
    s = traci.simulation.getTime()
    traci.simulationStep()
    return s


def print_remaining_plan(personID, comment=""):
    print("remaining stages for '%s' %s" % (personID, comment))
    for i in range(traci.person.getRemainingStages(personID)):
        print("  %s: %s" % (i, traci.person.getStage(personID, i)))


def print_prior_plan(personID, comment=""):
    print("prio stages for '%s' %s" % (personID, comment))
    stages = []
    i = -1
    while True:
        try:
            stages.append((i, traci.person.getStage(personID, i)))
            i -= 1
        except traci.TraCIException:
            break
    stages.reverse()
    for i, stage in stages:
        print("  %s: %s" % (i, stage))


def checkStateChanges():
    print("%s: departed=%s (%s) arrived=%s (%s)" % (
        traci.simulation.getTime(),
        traci.simulation.getDepartedPersonIDList(),
        traci.simulation.getDepartedPersonNumber(),
        traci.simulation.getArrivedPersonIDList(),
        traci.simulation.getArrivedPersonNumber()))


traci.start([sumolib.checkBinary('sumo'), "-c", "sumo.sumocfg", "--fcd-output",
             "fcd.xml"] + sys.argv[1:])
# add walking person
traci.person.add("newPerson", "3si", -10)
traci.person.appendWalkingStage("newPerson", ["3si", "2si"], -20)
traci.person.appendWaitingStage("newPerson", 10, "twiddling thumbs")
traci.person.appendWalkingStage("newPerson", ["2si", "2o"], 20)
traci.person.appendWaitingStage("newPerson", 20, "arrived")
traci.person.setSpeedFactor("newPerson", 1.3)

for i in range(3):
    checkStateChanges()
    print("step", step())


# test rerouting
print("detour edges", traci.person.getEdges("detour"))
traci.person.rerouteTraveltime("detour")
print("detour edges after routing", traci.person.getEdges("detour"))


# trip-like rerouting
traci.person.add("tripTest", "2fi", -10)
traci.person.appendWalkingStage("tripTest", ["2fi", "1fi"], -20)
traci.person.removeStage("tripTest", 0)  # proceed from initial waiting-to-depart stage
traci.person.rerouteTraveltime("tripTest")
print("tripTest edges after routing", traci.person.getEdges("tripTest"))

personID = "horiz"
traci.person.setType(personID, "pType2")
traci.person.setType(personID, "pType3")
traci.person.setLength(personID, 2)
traci.person.setHeight(personID, 1.9)
traci.person.setWidth(personID, 1.11)
traci.person.setMinGap(personID, 2.22)
traci.person.setColor(personID, (0, 0, 255, 255))
print("type", traci.person.getTypeID(personID))


def check(personID):
    print("persons", traci.person.getIDList())
    print("person count", traci.person.getIDCount())
    print("examining", personID)
    print("speed", traci.person.getSpeed(personID))
    print("speedFactor", traci.person.getSpeedFactor(personID))
    print("pos", traci.person.getPosition(personID))
    print("pos3D", traci.person.getPosition3D(personID))
    print("angle", traci.person.getAngle(personID))
    print("slope", traci.person.getSlope(personID))
    print("road", traci.person.getRoadID(personID))
    print("lane", traci.person.getLaneID(personID))
    print("type", traci.person.getTypeID(personID))
    print("lanePos", traci.person.getLanePosition(personID))
    print("color", traci.person.getColor(personID))
    print("length", traci.person.getLength(personID))
    print("MinGap", traci.person.getMinGap(personID))
    print("width", traci.person.getWidth(personID))
    print("waiting time", traci.person.getWaitingTime(personID))
    print("stage", traci.person.getStage(personID))
    print("remainingStages", traci.person.getRemainingStages(personID))
    print("edges", traci.person.getEdges(personID))
    print("vehicle", traci.person.getVehicle(personID))


check(personID)
traci.person.subscribe(personID)
print(traci.person.getSubscriptionResults(personID))
for i in range(3):
    print("step", step())
    print(traci.person.getSubscriptionResults(personID))

traci.person.setSpeedFactor(personID, 1.2)
check(personID)
try:
    check("bla")
except traci.TraCIException:
    print("recovering from exception after asking for unknown person")
print("step", step())

traci.person.removeStages("newPerson")
traci.person.appendDrivingStage("newPerson", "1o", "B42")

traci.route.add("r0", ["3si", "1o"])
traci.vehicle.addLegacy("veh0", "r0", traci.constants.DEPARTFLAG_TRIGGERED, pos=230)
traci.vehicle.setLine("veh0", "B42")
traci.vehicle.setStop("veh0", "3si", 235, laneIndex=2, startPos=230, duration=1)

print("getIDList", traci.person.getIDList())
print("numVehs=%s, numPersons=%s, minExpected=%s" % (
    traci.vehicle.getIDCount(),
    traci.person.getIDCount(),
    traci.simulation.getMinExpectedNumber()))

for i in range(10):
    print("step", step())
    print(traci.person.getSubscriptionResults(personID))

print("riding in vehicle: '%s'" % traci.vehicle.getParameter("veh0", "device.person.IDList"))
print("riding in vehicle (direct): '%s'" % traci.vehicle.getPersonIDList("veh0"))
print("persons on edge %s at time %s: %s" % (
    traci.person.getRoadID("newPerson"),
    traci.simulation.getTime(),
    traci.edge.getLastStepPersonIDs(traci.person.getRoadID("newPerson"))))

print_remaining_plan("newPerson", "remaining plan before jump")
traci.person.removeStages("newPerson")
traci.person.appendWaitingStage("newPerson", 10, "Jumped out of a moving vehicle. Ouch!")
print_prior_plan("newPerson", "past plan")
print_remaining_plan("newPerson", "remaining plan after jump")

step()
# change plan on junction
internalEdge = traci.person.getRoadID("detour")
print("'detour' on road", internalEdge)
traci.person.removeStages("detour")
traci.person.appendWalkingStage("detour", [internalEdge, "1si"], -20)

# reroute on junction
internalEdge = traci.person.getRoadID("detour2")
print("'detour2' on road", internalEdge)
print("detour2 edges", traci.person.getEdges("detour2"))
traci.person.rerouteTraveltime("detour2")
print("detour2 edges after routing", traci.person.getEdges("detour2"))

for i in range(196):
    step()
    if "detour2" in traci.person.getIDList():
        traci.person.rerouteTraveltime("detour2")

# person should be on a walkingArea in front of a crossing now
print(traci.person.getRoadID(personID))
print(traci.person.getLanePosition(personID))
print(traci.person.getNextEdge(personID))
# retrieve invalid stages
try:
    print(traci.person.getStage(personID, 3))
except traci.TraCIException:
    print("recovering from exception after asking for invalid stage index")
try:
    print(traci.person.getStage(personID, -2))
except traci.TraCIException:
    print("recovering from exception after asking for invalid stage index")

# changing walk edges in the middle of a walk
personTT2 = "tripTest2"
traci.person.add(personTT2, "2fi", 10)
traci.person.appendWalkingStage(personTT2, ["2fi"], -20)
for i in range(5):
    print("%s person=%s edge=%s pos=%s" % (
        traci.simulation.getTime(),
        personTT2,
        traci.person.getRoadID(personTT2),
        traci.person.getLanePosition(personTT2)))
    step()
traci.person.appendWalkingStage(personTT2, ["2fi", "1fi"], 10)
traci.person.removeStage(personTT2, 0)
print("  %s new edges edges=%s" % (personTT2, traci.person.getEdges(personTT2)))
for i in range(5):
    print("%s person=%s edge=%s pos=%s" % (
        traci.simulation.getTime(),
        personTT2,
        traci.person.getRoadID(personTT2),
        traci.person.getLanePosition(personTT2)))
    step()


# appendStage
traci.person.add("p3", "1fi", -10)
stage = traci.simulation.Stage(
    type=traci.constants.STAGE_WALKING,
    edges=["1fi", "1si"], departPos=-20, arrivalPos=10)
stage2 = traci.simulation.Stage(
    type=traci.constants.STAGE_WALKING,
    vType="car", edges=["1fi", "1o"], departPos=-20, arrivalPos=10)
stage3 = traci.simulation.Stage(
    type=traci.constants.STAGE_WALKING,
    vType="car", edges=["1o", "3o"], departPos=-20, arrivalPos=10)
stage4 = traci.simulation.Stage(
    type=traci.constants.STAGE_WALKING,
    vType="car", edges=["1o", "4o"], departPos=-20, arrivalPos=10)

traci.person.appendStage("p3", stage)
for i in range(10):
    traci.simulationStep()

remaining = traci.person.getRemainingStages("p3")
assert(remaining == 1)
# replace current stage
print_remaining_plan("p3", "(before replacement of current stage)")
traci.person.replaceStage("p3", 0, stage2)
print_remaining_plan("p3", "(after replacement)")
# replace later stage
traci.person.appendStage("p3", stage3)
print_remaining_plan("p3", "(before replacement of next stage)")
traci.person.replaceStage("p3", 1, stage4)
print_remaining_plan("p3", "(after replacement)")

print("speed (before setSpeed)", traci.person.getSpeed("p3"))
print("maxSpeed (before setSpeed)", traci.vehicletype.getMaxSpeed(traci.person.getTypeID("p3")))
traci.person.setSpeed("p3", 4.2)
print("speed (after setSpeed)", traci.person.getSpeed("p3"))
print("maxSpeed (after setSpeed)", traci.vehicletype.getMaxSpeed(traci.person.getTypeID("p3")))

for i in range(41):
    traci.simulationStep()
print("speed (at end)", traci.person.getSpeed("p3"))


traci.close()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
# Copyright (C) 2008-2022 German Aerospace Center (DLR) and others.
# This program and the accompanying materials are made available under the
# terms of the Eclipse Public License 2.0 which is available at
# https://www.eclipse.org/legal/epl-2.0/
# This Source Code may also be made available under the following Secondary
# Licenses when the conditions for such availability set forth in the Eclipse
# Public License 2.0 are satisfied: GNU General Public License, version 2
# or later which is available at
# https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
# SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later

# @file    runner.py
# @author  Michael Behrisch
# @author  Daniel Krajzewicz
# @date    2011-03-04


from __future__ import print_function
from __future__ import absolute_import
import os
import sys

if "SUMO_HOME" in os.environ:
    sys.path.append(os.path.join(os.environ["SUMO_HOME"], "tools"))
import traci  # noqa
import sumolib  # noqa


def printAggregated():
    print("lastTravelTime",
          traci.multientryexit.getLastIntervalMeanTravelTime(detID))
    print("lastHaltsPerVehicle",
          traci.multientryexit.getLastIntervalMeanHaltsPerVehicle(detID))
    print("lastTimeLoss",
          traci.multientryexit.getLastIntervalMeanTimeLoss(detID))
    print("lastVehicleSum",
          traci.multientryexit.getLastIntervalVehicleSum(detID))


traci.start([sumolib.checkBinary('sumo'), "-c", "sumo.sumocfg"])
for step in range(4):
    print("step", step)
    traci.simulationStep()
print("multientryexits", traci.multientryexit.getIDList())
print("multientryexit count", traci.multientryexit.getIDCount())
detID = "0"
print("examining", detID)
print("entryLanes", traci.multientryexit.getEntryLanes(detID))
print("exitLanes", traci.multientryexit.getExitLanes(detID))
print("entryPositions", traci.multientryexit.getEntryPositions(detID))
print("exitPositions", traci.multientryexit.getExitPositions(detID))
print("vehNum", traci.multientryexit.getLastStepVehicleNumber(detID))
print("meanSpeed", traci.multientryexit.getLastStepMeanSpeed(detID))
print("vehIDs", traci.multientryexit.getLastStepVehicleIDs(detID))
print("haltNum", traci.multientryexit.getLastStepHaltingNumber(detID))
printAggregated()

traci.multientryexit.setParameter(detID, "foo", "42")
print("parameter", traci.multientryexit.getParameter(detID, "foo"))
print("parameter from XML", traci.multientryexit.getParameter(detID, "loadedFromXML"))

traci.multientryexit.subscribe(detID)
print(traci.multientryexit.getSubscriptionResults(detID))
for step in range(3, 6):
    print("step", step)
    traci.simulationStep()
    print(traci.multientryexit.getSubscriptionResults(detID))

traci.simulationStep(110)
printAggregated()

traci.close()



#!/usr/bin/env python
# Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
# Copyright (C) 2008-2022 German Aerospace Center (DLR) and others.
# This program and the accompanying materials are made available under the
# terms of the Eclipse Public License 2.0 which is available at
# https://www.eclipse.org/legal/epl-2.0/
# This Source Code may also be made available under the following Secondary
# Licenses when the conditions for such availability set forth in the Eclipse
# Public License 2.0 are satisfied: GNU General Public License, version 2
# or later which is available at
# https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
# SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later

# @file    runner.py
# @author  Jakob Erdmann
# @author  Laura Bieker
# @date    2013-10-07

from __future__ import absolute_import

import os
import sys

if 'SUMO_HOME' in os.environ:
    sys.path.append(os.path.join(os.environ["SUMO_HOME"], "tools"))
import sumolib  # noqa
import traci  # noqa


cmd = [sumolib.checkBinary('sumo'),
       "-n", "input_net.net.xml",
       "-r", "input_routes.rou.xml",
       "-a", "input_additional.add.xml",
       "--no-step-log", "-S", "-Q"] + sys.argv[1:]


def run():
    """execute the TraCI control loop"""
    traci.start(cmd)
    step = 0
    while traci.simulation.getMinExpectedNumber() > 0 and step < 100:
        traci.simulationStep()
        step += 1
        if step == 4:
            traci.trafficlight.setProgram("center", "0")
    traci.close()
    sys.stdout.flush()


run()



#!/usr/bin/env python
# Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
# Copyright (C) 2017-2022 German Aerospace Center (DLR) and others.
# This program and the accompanying materials are made available under the
# terms of the Eclipse Public License 2.0 which is available at
# https://www.eclipse.org/legal/epl-2.0/
# This Source Code may also be made available under the following Secondary
# Licenses when the conditions for such availability set forth in the Eclipse
# Public License 2.0 are satisfied: GNU General Public License, version 2
# or later which is available at
# https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
# SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later

# @file    test.py
# @author  Leonhard Luecken
# @date    2017

import unittest as ut
import os
import sys

# Put tools into PYTHONPATH
if "SUMO_HOME" in os.environ:
    sys.path.append(os.path.join(os.environ["SUMO_HOME"], "tools"))

import traci  # noqa
import simpla  # noqa
import sumolib  # noqa
import simpla._reporting as rp  # noqa
import simpla._config as cfg  # noqa
from simpla._platoonmode import PlatoonMode  # noqa
from functools import reduce  # noqa


class TestPlatoonManager(ut.TestCase):

    def setUp(self):
        print("\n")
        ut.TestCase.setUp(self)
        testDir = os.path.dirname(os.path.realpath(__file__))
        # Declare simpla config files
        self.CFG0 = 'simpla.cfg.xml'
        self.CFG1 = 'simpla_test.cfg.xml'

        self.SIMPLA_CFG = os.path.join(testDir, "simpla.cfg.xml")
        self.SIMPLA_CFG_WARN = os.path.join(testDir, "simpla_test_warn.cfg.xml")
        self.SIMPLA_CFG_VTYPEMAP = os.path.join(testDir, "simpla_test_vtypemap.cfg.xml")

        # define config contents
        self.cfg_body0 =\
            """
                <vTypeMapFile value="vtype.map" />
                <controlRate value="10." />
                <vehicleSelectors value="connected" />
                <maxPlatoonGap value="15.0" />
                <catchupDist value="50.0" />
                <switchImpatienceFactor value="0.1" />
                <platoonSplitTime value="3.0" />
                <useHeadway value="false" />
                <lcMode original="597" leader="597" follower="514" catchup="514" catchupFollower="514" />
                <speedFactor original="1.01" leader="1.01" follower="1.11" catchup="1.21" catchupFollower="1.31" >\
</speedFactor>
                <verbosity value="2" />
            """

        self.cfg_body1 =\
            """
                <vTypeMapFile file="vtype.map"/>
                <controlRate value="1000." />
                <vehicleSelectors value="connected" />
                <maxPlatoonGap value="15.0" />
                <catchupDist value="50.0" />
                <switchImpatienceFactor value="0.1" />
                <platoonSplitTime value="3.0" />
                <useHeadway value="false" />
                <lcMode original="597" leader="597" follower="514" catchup="514" catchupFollower="514" />
                <speedFactor original="1.01" leader="1.01" follower="1.11" catchup="1.21" catchupFollower="1.31" >\
</speedFactor>
                <verbosity value="2" />
            """

        self.cfg_body2 =\
            """
                <vTypeMap original="unknownVTypeID" leader="leaderVTypeID" follower="followerVTypeID" \
catchup="catchupVTypeID" catchupFollower="catchupFollowerVTypeID" />
                <useHeadway value="false" />
            """

        self.cfg_body3 =\
            """
                <controlRate value="10." />
                <vehicleSelectors value="connected" />
                <maxPlatoonGap value="15.0" />
                <catchupDist value="50.0" />
                <switchImpatienceFactor value="0.1" />
                <platoonSplitTime value="3.0" />
                <useHeadway value="false" />
                <lcMode original="597" leader="597" follower="514" catchup="514" catchupFollower="514" />
                <speedFactor original="1.01" leader="1.01" follower="1.11" catchup="1.21" catchupFollower="1.31" >\
</speedFactor>
                <verbosity value="2" />
                <vTypeMap original="connected" leader="connected_pLeader" follower="connected_pFollower" \
catchup="connected_pCatchup" catchupFollower="connected_pCatchupFollower" />
            """

        self.cfg_body4 =\
            """
                <vTypeMap original="connected" leader="connected_pLeader" follower="connected_pFollower"/>
                <useHeadway value="false" />
            """
        # template still needs to insert definite values for placeholders
        self.SUMO_CFG_TEMPLATE = os.path.join(testDir, "sumo.sumocfg")
        self.SUMO_CFG = os.path.join(testDir, "test.sumocfg")
        self.patchSumoConfig()  # write default routes file and net file into config

    def connectToSumo(self, sumo_cfg):
        traci.start([sumolib.checkBinary('sumo'), "-c", sumo_cfg])

    def patchSumoConfig(self, routes_fn="input_routes.rou.xml", net_fn="input_net.net.xml",
                        vtypes_fn="input_types.typ.xml"):
        # replace routes_fn in config template
        with open(self.SUMO_CFG_TEMPLATE, "r") as cfg_template, open(self.SUMO_CFG, "w") as cfg:
            template_str = cfg_template.read()
            cfg_str = template_str.format(routes_file=routes_fn, net_file=net_fn, vtypes_file=vtypes_fn)
            cfg.write(cfg_str)

    def patchConfigFile(self, cfg_body):
        # patch simpla config
        with open(self.CFG0, "r") as empty_cfg, open(self.CFG1, "w") as target_cfg:
            s = empty_cfg.read()
            # print(s.format(body=cfg_body))
            target_cfg.write(s.format(body=cfg_body))

    def tearDown(self):
        ut.TestCase.tearDown(self)
        # stop simpla
        simpla.stop()
        # close TraCI connection
        traci.close()
        # reset simpla
        rp.initDefaults()
        cfg.initDefaults()
        if os.path.exists(self.SUMO_CFG):
            os.remove(self.SUMO_CFG)

    def test_init(self):
        print("Testing platoon manager initialization...")
        self.patchSumoConfig()
        self.connectToSumo(self.SUMO_CFG)
        self.patchConfigFile(self.cfg_body0)
        simpla.load(self.CFG1)
        self.assertListEqual([r for t, r in rp.WARNING_LOG], [])

        expectedVTypes = ["connected", "connected_pLeader", "connected_pFollower",
                          "connected_pCatchup", "connected_pCatchupFollower"]
        registeredPlatoonVTypes = list(
            set(reduce(lambda x, y: x + y, [[orig] + list(mapped.values())
                                            for orig, mapped in cfg.PLATOON_VTYPES.items()])))
        expectedVTypes.sort()
        registeredPlatoonVTypes.sort()
        self.assertListEqual(expectedVTypes, registeredPlatoonVTypes)

    def test_init_vtypemap(self):
        print("Testing specification per vtypemap xml-element...")
        self.patchSumoConfig()
        self.connectToSumo(self.SUMO_CFG)
        self.patchConfigFile(self.cfg_body3)
        simpla.load(self.CFG1)
#         simpla.load(self.SIMPLA_CFG_VTYPEMAP)
        self.assertListEqual([r for t, r in rp.WARNING_LOG], [])

        expectedVTypes = ["connected", "connected_pLeader", "connected_pFollower",
                          "connected_pCatchup", "connected_pCatchupFollower"]
        registeredPlatoonVTypes = list(
            set(reduce(lambda x, y: x + y, [[orig] + list(mapped.values())
                                            for orig, mapped in cfg.PLATOON_VTYPES.items()])))
        expectedVTypes.sort()
        registeredPlatoonVTypes.sort()
        self.assertListEqual(expectedVTypes, registeredPlatoonVTypes)

    def test_init_vtypemap_fallback(self):
        print("Testing specification per vtypemap xml-element with missing modes...")
        self.patchSumoConfig()
        self.connectToSumo(self.SUMO_CFG)
        self.patchConfigFile(self.cfg_body4)
        simpla.load(self.CFG1)

    def test_init_warn(self):
        print("Testing Warnings...")
        self.patchSumoConfig(vtypes_fn="input_types2.typ.xml")
        self.connectToSumo(self.SUMO_CFG)
        # print (self.cfg_body1)
        self.patchConfigFile(self.cfg_body1)
        simpla.load(self.CFG1)
        # simpla.load(self.SIMPLA_CFG_WARN)
        expected_warnings = [
            "WARNING: Restricting given control rate (= 1000 per sec.) to 1 per timestep (= 10 per sec.) " +
            "(PlatoonManager)",
            "WARNING: emergencyDecel of mapped vType 'connected_pCatchupFollower' (10.5m.) does not equal " +
            "emergencyDecel of original vType 'connected' (4.5m.) (PlatoonManager)",
            "WARNING: emergencyDecel of mapped vType 'connected_pFollower' (1.7m.) does not equal emergencyDecel " +
            "of original vType 'connected' (4.5m.) (PlatoonManager)",
            "WARNING: emergencyDecel of mapped vType 'connected_pCatchup' (0.5m.) does not equal emergencyDecel of " +
            "original vType 'connected' (4.5m.) (PlatoonManager)",
            "WARNING: length of mapped vType 'connected_pLeader' (10.0m.) does not equal length of original vType " +
            "'connected' (5.0m.)\nThis will probably lead to collisions. (PlatoonManager)",
            "WARNING: length of mapped vType 'connected_pCatchupFollower' (3.0m.) does not equal length of original " +
            "vType 'connected' (5.0m.)\nThis will probably lead to collisions. (PlatoonManager)",
        ]
        warnings_list = [r for t, r in rp.WARNING_LOG]
        # for w in warnings_list:
        #    print(w)
        for w in expected_warnings:
            self.assertIn(w, warnings_list)
        self.assertListEqual([], list(set(warnings_list).difference(expected_warnings)))
        traci.simulationStep(1.)
        self.assertEqual(rp.WARNING_LOG[-1][0], "1.0")
        self.assertEqual(
            rp.WARNING_LOG[-1][1], "WARNING: Step lengths that differ from SUMO's simulation step length are not " +
            "supported and probably lead to undesired behavior.\nConsider decreasing simpla's control rate instead. " +
            "(PlatoonManager)")

    def test_unknown_vtypes(self):
        print("Testing Exceptions for unknown vTypes...")
        self.patchSumoConfig(vtypes_fn="input_types2.typ.xml")
        self.connectToSumo(self.SUMO_CFG)
        # print (self.cfg_body1)
        self.patchConfigFile(self.cfg_body2)
        try:
            simpla.load(self.CFG1)
            self.assertTrue(False, "PlatoonManager() should raise an exception in case of unknown vtypes")
        except simpla.SimplaException as e:
            self.assertEqual(
                str(e), "vType 'unknownVTypeID' is unknown to sumo! Note: Platooning vTypes must be defined at " +
                "startup.")

    def test_add_and_remove(self):
        print("Testing adding and removing connected vehicles...")
        self.patchSumoConfig()
        self.connectToSumo(self.SUMO_CFG)

        self.patchConfigFile(self.cfg_body0)
        simpla.load(self.CFG1)
#         simpla.load(self.SIMPLA_CFG)
        mgr = simpla._mgr

#         # load simpla without adding a step listener
#         simpla._config.load(self.SIMPLA_CFG)
#         simpla._mgr = simpla._platoonmanager.PlatoonManager()

        self.assertTupleEqual((), traci.vehicle.getIDList())

        traci.simulationStep()
        self.assertTupleEqual(('connected.1',), traci.vehicle.getIDList())
        self.assertListEqual(['connected.1'], [vehID for vehID in mgr._connectedVehicles.keys()])

        self.assertEqual(rp.REPORT_LOG[-1][0], "0.1")
        self.assertEqual(rp.REPORT_LOG[-1][1], "Adding vehicle 'connected.1' (PlatoonManager)")

        while traci.simulation.getTime() < 2:
            traci.simulationStep()

        self.assertListEqual(list(sorted(['connected.1', 'conventional.1'])), list(sorted(traci.vehicle.getIDList())))
        self.assertListEqual(['connected.1'], [vehID for vehID in mgr._connectedVehicles.keys()])

        while traci.simulation.getTime() <= 5:
            traci.simulationStep()

        self.assertEqual(rp.REPORT_LOG[-1][0], "3.1")
        self.assertEqual(rp.REPORT_LOG[-1][1], "Adding vehicle 'IAMconnectedTOO' (PlatoonManager)")

        self.assertTrue(mgr._hasConnectedType("IAMconnectedTOO"))
        self.assertListEqual(list(sorted(['connected.1', 'conventional.1', 'IAMconnectedTOO'])), list(
            sorted(traci.vehicle.getIDList())))
        self.assertListEqual(list(sorted(['connected.1', 'IAMconnectedTOO'])), list(
            sorted([vehID for vehID in mgr._connectedVehicles.keys()])))

        while traci.simulation.getTime() <= 14:
            traci.simulationStep()

        self.assertEqual(rp.REPORT_LOG[-1][0], "11.4")
        self.assertEqual(
            rp.REPORT_LOG[-1][1], "Platoon '1' joined Platoon '0', which now contains vehicles:\n['connected.1', " +
            "'IAMconnectedTOO'] (PlatoonManager)")

        while traci.simulation.getTime() <= 17:
            traci.simulationStep()

        self.assertEqual(rp.REPORT_LOG[-1][0], "16.0")
        self.assertEqual(
            rp.REPORT_LOG[-1][1], "Platoon '0' splits (ID of new platoon: '2'):\n    Platoon '0': ['connected.1']\n" +
            "    Platoon '2': ['IAMconnectedTOO'] (PlatoonManager)")

        while traci.simulation.getTime() <= 18:
            traci.simulationStep()

        self.assertEqual(rp.REPORT_LOG[-1][0], "17.3")
        self.assertEqual(rp.REPORT_LOG[-1][1], "Removing arrived vehicle 'connected.1' (PlatoonManager)")
        self.assertListEqual(list(sorted(['conventional.1', 'IAMconnectedTOO'])),
                             list(sorted(traci.vehicle.getIDList())))
        self.assertListEqual(['IAMconnectedTOO'], [vehID for vehID in mgr._connectedVehicles.keys()])

        while traci.simulation.getTime() <= 20:
            traci.simulationStep()

        self.assertEqual(rp.REPORT_LOG[-1][0], "19.8")
        self.assertEqual(rp.REPORT_LOG[-1][1], "Removing arrived vehicle 'IAMconnectedTOO' (PlatoonManager)")

    def test_platoon_formation(self):
        print("Testing platoon formation...")
        self.patchSumoConfig(net_fn="input_net2.net.xml", routes_fn="input_routes2.rou.xml")
        self.connectToSumo(self.SUMO_CFG)

        self.patchConfigFile(self.cfg_body0)
        simpla.load(self.CFG1)
#         simpla.load(self.SIMPLA_CFG)
        mgr = simpla._mgr

        while traci.simulation.getTime() <= 5:
            traci.simulationStep()

        self.assertIn("connected.1", mgr._connectedVehicles)
        self.assertIn("connected.2", mgr._connectedVehicles)
        veh1 = mgr._connectedVehicles["connected.1"]
        self.assertEqual(veh1.getCurrentPlatoonMode(), PlatoonMode.NONE)
        veh2 = mgr._connectedVehicles["connected.2"]
        self.assertEqual(veh2.getCurrentPlatoonMode(), PlatoonMode.CATCHUP)

        while traci.simulation.getTime() <= 20:
            traci.simulationStep()

#         self.assertEqual(rp.REPORT_LOG[-1][0], "13.7")
#         self.assertEqual(rp.REPORT_LOG[-1][1], "Platoon '1' joined Platoon '0', which now contains " +
#                          "vehicles:\n['connected.1', 'connected.2'] (PlatoonManager)")

        self.assertEqual(veh1.getCurrentPlatoonMode(), PlatoonMode.LEADER)
        self.assertEqual(veh2.getCurrentPlatoonMode(), PlatoonMode.FOLLOWER)

        vehs = mgr.getPlatoonLeaders()[0].getPlatoon().getVehicles()
        vehIDs = [v.getID() for v in vehs]
        self.assertEqual(len(vehIDs), 2)
        self.assertIn("connected.1", vehIDs)
        self.assertIn("connected.2", vehIDs)

        while traci.vehicle.getLaneID(vehIDs[0]) == traci.vehicle.getLaneID(vehIDs[1]):
            traci.simulationStep()

        self.assertFalse(veh1.state.laneID == veh2.state.laneID)
        t0 = traci.simulation.getTime()
        self.assertEqual(t0, 71.4)

        expected_split_time = t0 + mgr._DeltaT + cfg.PLATOON_SPLIT_TIME

        while traci.simulation.getTime() <= expected_split_time:
            traci.simulationStep()

        self.assertAlmostEqual(veh2._timeUntilSplit, 0.0, 9)

        while traci.simulation.getTime() <= expected_split_time + 1.:
            traci.simulationStep()

        self.assertEqual(
            rp.REPORT_LOG[-1][1], "Platoon '0' splits (ID of new platoon: '6'):\n    Platoon '0': ['connected.1']\n" +
            "    Platoon '6': ['connected.2'] (PlatoonManager)")


# # ignore some tests for faster execution
# ignore_tests = [
#     "test_init",
#     "test_init_vtypemap",
# #     "test_init_warn",
#     "test_add_and_remove",
#     "test_platoon_formation",
#     ]
#
# for t in ignore_tests:
#     delattr(TestPlatoonManager,t)


if __name__ == "__main__":
    ut.main()



#!/usr/bin/env python
# Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
# Copyright (C) 2020-2022 German Aerospace Center (DLR) and others.
# This program and the accompanying materials are made available under the
# terms of the Eclipse Public License 2.0 which is available at
# https://www.eclipse.org/legal/epl-2.0/
# This Source Code may also be made available under the following Secondary
# Licenses when the conditions for such availability set forth in the Eclipse
# Public License 2.0 are satisfied: GNU General Public License, version 2
# or later which is available at
# https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
# SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later

# @file    runner.py
# @author  Michael Behrisch
# @date    2020-09-21

from __future__ import print_function
import os
import sys
import subprocess
sys.path.append(os.path.join(os.environ["SUMO_HOME"], "tools"))
import sumolib  # noqa

numLanes = int(sys.argv[1]) if len(sys.argv) > 1 else 60
length = float(sys.argv[2]) if len(sys.argv) > 2 else 1e5
with open("%s.edg.xml" % numLanes, "w") as edges, open("%s.nod.xml" % numLanes, "w") as nodes, \
     open("%s.rou.xml" % numLanes, "w") as flows:
    print("<edges>", file=edges)
    print("<nodes>", file=nodes)
    print("<routes>", file=flows)
    for i in range(numLanes):
        print('    <node id="s%s" x="0" y="%s"/>' % (i, i*10), file=nodes)
        print('    <node id="t%s" x="%s" y="%s"/>' % (i, length, i*10), file=nodes)
        print('    <edge id="%s" from="s%s" to="t%s"/>' % (i, i, i), file=edges)
        print('    <flow id="%s" begin="0.00" end="100" period="2"><route edges="%s"/></flow>' % (i, i), file=flows)
    print("</edges>", file=edges)
    print("</nodes>", file=nodes)
    print("</routes>", file=flows)
subprocess.call([sumolib.checkBinary("netconvert"), "-o", "%s.net.xml" % numLanes, "-n", nodes.name, "-e", edges.name])
# generate config
subprocess.call([sumolib.checkBinary("sumo"), "-n", "%s.net.xml" % numLanes, "-r", flows.name, "--no-step-log",
                 "-C", "%s.sumocfg" % numLanes])
# run single threaded
subprocess.call([sumolib.checkBinary("sumo"), "-c", "%s.sumocfg" % numLanes])
# run with 2 threads
subprocess.call([sumolib.checkBinary("sumo"), "-c", "%s.sumocfg" % numLanes, "--threads", "2"])



#!/usr/bin/env python
# Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
# Copyright (C) 2008-2022 German Aerospace Center (DLR) and others.
# This program and the accompanying materials are made available under the
# terms of the Eclipse Public License 2.0 which is available at
# https://www.eclipse.org/legal/epl-2.0/
# This Source Code may also be made available under the following Secondary
# Licenses when the conditions for such availability set forth in the Eclipse
# Public License 2.0 are satisfied: GNU General Public License, version 2
# or later which is available at
# https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
# SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later

# @file    runner.py
# @author  Michael Behrisch
# @date    2021-01-13

import os
import subprocess
import sys
sys.path.append(os.path.join(os.environ["SUMO_HOME"], "tools"))
import sumolib  # noqa


subprocess.call([sumolib.checkBinary("netgenerate"), "--grid", "-o", "test.net.xml"])
large = 2**20 * " "
with open("test.net.xml", "a") as net:
    for _ in range(4 * 2**10):
        net.write(large)
subprocess.call([sumolib.checkBinary('sumo'), "-n", "test.net.xml", "--no-step-log"])
os.remove("test.net.xml")



#!/usr/bin/env python
# Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
# Copyright (C) 2008-2022 German Aerospace Center (DLR) and others.
# This program and the accompanying materials are made available under the
# terms of the Eclipse Public License 2.0 which is available at
# https://www.eclipse.org/legal/epl-2.0/
# This Source Code may also be made available under the following Secondary
# Licenses when the conditions for such availability set forth in the Eclipse
# Public License 2.0 are satisfied: GNU General Public License, version 2
# or later which is available at
# https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
# SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later

# @file    runner.py
# @author  Daniel Krajzewicz
# @author  Michael Behrisch
# @author  Jakob Erdmann
# @date    2012-08-10

from __future__ import absolute_import
from __future__ import print_function

import os
import subprocess
import sys
sys.path.append(os.path.join(os.environ["SUMO_HOME"], "tools"))
import sumolib  # noqa


sumoBinary = sumolib.checkBinary('sumo')


def call(comment, args):
    print(comment, args)
    sys.stdout.flush()
    subprocess.call([sumoBinary, "-c", "sumo.sumocfg", "-C", "stdout"] + args, stderr=subprocess.STDOUT)
    print("-" * 10)


call("plain", [])
call("replace", ["-a", "new.rou.xml"])
call("replace", ["--a", "new.rou.xml"])
call("append+", ["+a", "new.rou.xml"])
call("append++", ["++a", "new.rou.xml"])
call("append+-", ["+-a", "new.rou.xml"])
call("append-+", ["-+a", "new.rou.xml"])
call("replace", ["-additional", "new.rou.xml"])
call("replace", ["--additional", "new.rou.xml"])
call("append+", ["+additional", "new.rou.xml"])
call("append++", ["++additional", "new.rou.xml"])
call("append+-", ["+-additional", "new.rou.xml"])
call("append-+", ["-+additional", "new.rou.xml"])

call("replace", ["-a=new.rou.xml"])
call("replace", ["--a=new.rou.xml"])
call("append+", ["+a=new.rou.xml"])
call("append++", ["++a=new.rou.xml"])
call("append+-", ["+-a=new.rou.xml"])
call("append-+", ["-+a=new.rou.xml"])
call("replace", ["-additional=new.rou.xml"])
call("replace", ["--additional=new.rou.xml"])
call("append+", ["+additional=new.rou.xml"])
call("append++", ["++additional=new.rou.xml"])
call("append+-", ["+-additional=new.rou.xml"])
call("append-+", ["-+additional=new.rou.xml"])



#!/usr/bin/env python
# Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
# Copyright (C) 2008-2022 German Aerospace Center (DLR) and others.
# This program and the accompanying materials are made available under the
# terms of the Eclipse Public License 2.0 which is available at
# https://www.eclipse.org/legal/epl-2.0/
# This Source Code may also be made available under the following Secondary
# Licenses when the conditions for such availability set forth in the Eclipse
# Public License 2.0 are satisfied: GNU General Public License, version 2
# or later which is available at
# https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
# SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later

# @file    runner.py
# @author  Michael Behrisch
# @date    2011-06-15


from __future__ import print_function
from __future__ import absolute_import
import os
import subprocess
import sys
from xml.sax import parse, handler
from collections import defaultdict
if "SUMO_HOME" in os.environ:
    sys.path.append(os.path.join(os.environ["SUMO_HOME"], "tools"))
import sumolib  # noqa


class OutputHandler(handler.ContentHandler):

    def __init__(self, lanes):
        self.lanes = lanes
        self.speed = dict([(la, defaultdict(dict)) for la in lanes])
        self.intervals = set()

    def startElement(self, name, attrs):
        if "id" in attrs:
            lane = attrs["id"][3:]
        if name == "interval":
            self.interval = (float(attrs["begin"]), float(attrs["end"]))
            self.intervals.add(self.interval)
            if "flow" in attrs:
                self.speed[lane]["e1"][self.interval] = float(attrs["speed"])
            if "meanMaxJamLengthInVehicles" in attrs:
                self.speed[lane]["e2"][
                    self.interval] = float(attrs["meanSpeed"])
            if "meanSpeedWithin" in attrs:
                self.speed[lane]["e3"][self.interval] = float(
                    attrs["meanSpeedWithin"])
                if self.speed[lane]["e3"][self.interval] == -1.:
                    self.speed[lane]["e3"][
                        self.interval] = float(attrs["meanSpeed"])
        if name == "edge" and "id" in attrs and attrs["id"] + "_0" in self.lanes and "speed" in attrs:
            self.speed[
                attrs["id"] + "_0"]["edge"][self.interval] = float(attrs["speed"])
        if name == "lane" and "id" in attrs and attrs["id"] in self.lanes and "speed" in attrs:
            self.speed[attrs["id"]]["lane"][
                self.interval] = float(attrs["speed"])


def generateDetectorDef(out, freq, enableLoop, laneIDs):
    print(
        '<additional xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:noNamespaceSchemaLocation=' +
        '"http://sumo.dlr.de/xsd/additional_file.xsd">', file=out)
    for laneId in laneIDs:
        if enableLoop:
            print('    <e1Detector id="e1_%s" lane="%s" pos="200" freq="%s" file="detector.xml"/>' %
                  (laneId, laneId, freq), file=out)
        print("""    <e2Detector id="e2_%s" lane="%s" pos="0" length="30000" friendlyPos="true" freq="%s" file="detector.xml"/>
        <e3Detector id="e3_%s" freq="%s" file="detector.xml">
            <detEntry lane="%s" pos="0"/>
            <detExit lane="%s" pos="30000" friendlyPos="true"/>
        </e3Detector>""" % (laneId, laneId, freq, laneId, freq, laneId, laneId), file=out)
    print("""    <edgeData id="dump_15" freq="%s" file="meandataedge.xml" excludeEmpty="true"/>
    <laneData id="dump_15" freq="%s" file="meandatalane.xml" excludeEmpty="true"/>
</additional>""" % (freq, freq), file=out)


def checkOutput(freq, args, withLoop, lanes):
    handler = OutputHandler(lanes)
    for f in ["detector.xml", "meandataedge.xml", "meandatalane.xml"]:
        if os.path.exists(f):
            parse(f, handler)
    for i in sorted(handler.intervals):
        for lane in lanes:
            types = ["e1", "e2", "e3", "edge", "lane"]
            if not withLoop:
                types = types[1:]
            vals = [handler.speed[lane][type].get(i, -1.) for type in types]
            for v in vals[:-1]:
                if abs(v - vals[-1]) > 0.001:
                    print("failed", freq, args, lane, i, zip(types, vals))
                    return
    print("success", freq, args, lanes)


def flush():
    sys.stdout.flush()
    sys.stderr.flush()


sumoBinary = sumolib.checkBinary('sumo')
sumoArgStart = len(sys.argv)
for idx, arg in enumerate(sys.argv):
    if arg[0] == "-":
        sumoArgStart = idx
        break
lanes = sys.argv[1:sumoArgStart]
for stepLength in [".1", "1"]:
    for end in [51, 100, 1000]:
        args = sys.argv[sumoArgStart:] + \
            ["--step-length", stepLength, "--end", str(end)]
        for freq in [.1, 1, 10, 100]:
            withLoop = freq > 55 and end > 55
            for numLanes in range(1, len(lanes) + 1):
                with open("input_additional.add.xml", 'w') as out:
                    generateDetectorDef(out, freq, withLoop, lanes[:numLanes])
                exitCode = subprocess.call(
                    [sumoBinary, "-c", "sumo.sumocfg"] + args)
                flush()
                if exitCode:
                    sys.exit(exitCode)
                checkOutput(freq, args, withLoop, lanes[:numLanes])
                flush()



#!/usr/bin/env python
# Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
# Copyright (C) 2008-2022 German Aerospace Center (DLR) and others.
# This program and the accompanying materials are made available under the
# terms of the Eclipse Public License 2.0 which is available at
# https://www.eclipse.org/legal/epl-2.0/
# This Source Code may also be made available under the following Secondary
# Licenses when the conditions for such availability set forth in the Eclipse
# Public License 2.0 are satisfied: GNU General Public License, version 2
# or later which is available at
# https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
# SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later

# @file    runner.py
# @author  Daniel Krajzewicz
# @author  Michael Behrisch
# @date    2010-03-21

from __future__ import absolute_import
from __future__ import print_function

import os
import subprocess
import sys

if "SUMO_HOME" in os.environ:
    sys.path.append(os.path.join(os.environ["SUMO_HOME"], "tools"))
import sumolib  # noqa
import traci  # noqa

PORT = sumolib.miscutils.getFreeSocketPort()
DELTA_T = 1000

sumoBinary = sumolib.checkBinary(sys.argv[1])
if sys.argv[1] == "sumo":
    addOption = []
else:
    addOption = ["-S", "-Q"]


def runSingle(traciEndTime, sumoEndTime=None):
    step = 0
    if sumoEndTime is None:
        opt = addOption
    else:
        opt = addOption + ["--end", str(sumoEndTime)]
    sumoProcess = subprocess.Popen([sumoBinary, "-c", "sumo.sumocfg", "--remote-port", str(PORT)] + opt,
                                   stdout=sys.stdout)
    traci.init(PORT)
    while not step > traciEndTime:
        traci.simulationStep()
        step += 1
    print("Print ended at step", traci.simulation.getTime())
    traci.close()
    sumoProcess.wait()
    sys.stdout.flush()


print("=========== long route ===========")
fdo = open("input_routes.rou.xml", "w")
print('<routes>', file=fdo)
print(
    '   <route id="horizontal" edges="2fi 2si 1o 1fi 1si 3o 3fi 3si 4o 4fi 4si"/>', file=fdo)
print('   <vehicle id="horiz" route="horizontal" depart="0"/>', file=fdo)
print('</routes>', file=fdo)
fdo.close()
print("----------- SUMO end time is smaller than TraCI's -----------")
sys.stdout.flush()
runSingle(99, 50)
print("----------- SUMO end time is not given -----------")
sys.stdout.flush()
runSingle(99)


print("=========== empty routes in SUMO ===========")
fdo = open("input_routes.rou.xml", "w")
print('<routes>', file=fdo)
print('</routes>', file=fdo)
fdo.close()
print("----------- SUMO end time is smaller than TraCI's -----------")
sys.stdout.flush()
runSingle(99, 50)
print("----------- SUMO end time is not given -----------")
sys.stdout.flush()
runSingle(99)


print("=========== vehicle leaves before TraCI ends ===========")
fdo = open("input_routes.rou.xml", "w")
print('<routes>', file=fdo)
print('   <route id="horizontal" edges="2fi 2si"/>', file=fdo)
print('   <vehicle id="horiz" route="horizontal" depart="0"/>', file=fdo)
print('</routes>', file=fdo)
fdo.close()
print("----------- SUMO end time is smaller than TraCI's -----------")
sys.stdout.flush()
runSingle(99, 50)
print("----------- SUMO end time is not given -----------")
sys.stdout.flush()
runSingle(99)



# Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
# Copyright (C) 2012-2022 German Aerospace Center (DLR) and others.
# This program and the accompanying materials are made available under the
# terms of the Eclipse Public License 2.0 which is available at
# https://www.eclipse.org/legal/epl-2.0/
# This Source Code may also be made available under the following Secondary
# Licenses when the conditions for such availability set forth in the Eclipse
# Public License 2.0 are satisfied: GNU General Public License, version 2
# or later which is available at
# https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
# SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later

# @file    evaluator.py
# @author  Daniel Krajzewicz
# @date    13-06-07


from __future__ import absolute_import
from __future__ import print_function

import sumolib.output
from pylab import clf, colorbar, imshow, legend, savefig, title, xticks, yticks
from runner import flow1def, flow2def, types


durationM = {}
waitStepsM = {}
durationMinMax = [0, 0]
waitStepsMinMax = [0, 0]
f1range = range(int(flow1def[0]), int(flow1def[1]), int(flow1def[2]))
rf1range = range(int(flow1def[0]), int(flow1def[1]), int(flow1def[2]))
rf1range.reverse()
f2range = range(int(flow2def[0]), int(flow2def[1]), int(flow2def[2]))
rf2range = range(int(flow2def[0]), int(flow2def[1]), int(flow2def[2]))
rf2range.reverse()
for t in types:
    print("Processing outputs for %s" % t)
    durationM[t] = []
    waitStepsM[t] = []
    for f1 in rf1range:
        print(" f1 at %s" % f1)
        durationM[t].append([])
        waitStepsM[t].append([])
        for f2 in f2range:
            duration = 0
            waitSteps = 0
            vehNum = 0
            # summary
            pd = sumolib.output.parse(
                "results/tripinfos_%s_%s_%s.xml" % (t, f1, f2), "tripinfo")
            for v in pd:
                if float(v.depart) < 3600:
                    continue
                duration = duration + float(v.duration)
                waitSteps = waitSteps + float(v.waitSteps)
                vehNum = vehNum + 1
            if vehNum != 0:
                duration = duration / float(vehNum)
                waitSteps = waitSteps / float(vehNum)
            durationM[t][-1].append(duration)
            waitStepsM[t][-1].append(waitSteps)
            if duration > durationMinMax[1]:
                durationMinMax[1] = duration
            if waitSteps > waitStepsMinMax[1]:
                waitStepsMinMax[1] = waitSteps

    """
   <interval begin="0.00" end="60.00" id="2i_l0" nSamples="55"
   meanSpeed="13.89" meanOccupancy="1.19" maxOccupancy="3.90"
   meanMaxJamLengthInVehicles="0.00" meanMaxJamLengthInMeters="0.00" maxJamLengthInVehicles="0"
   maxJamLengthInMeters="0.00" jamLengthInVehiclesSum="0" jamLengthInMetersSum="0.00" meanHaltingDuration="0.00"
   maxHaltingDuration="0.00" haltingDurationSum="0.00" meanIntervalHaltingDuration="0.00"
   maxIntervalHaltingDuration="0.00" intervalHaltingDurationSum="0.00"
   startedHalts="0.00" meanVehicleNumber="0.92" maxVehicleNumber="3" />
    """


def makeIMSHOWfigure(matrix, oname, t, rangeX, rangeY, minMax=None):
    if minMax:
        imshow(matrix, vmin=minMax[0], vmax=minMax[1], interpolation='nearest')
    else:
        imshow(matrix, interpolation='nearest')
    legend()
    colorbar(shrink=0.5)
    xticks(range(0, len(matrix)), rangeX, size=14)
    yticks(range(0, len(matrix[0])), rangeY, size=14)
    title(t)
    savefig(oname)
    clf()


for t in types:
    makeIMSHOWfigure(durationM[t], "durationSS_%s_png" %
                     t, "average travel time\n(%s)" % t, f1range, rf2range, durationMinMax)
    makeIMSHOWfigure(waitStepsM[t], "waitStepsSS_%s_png" %
                     t, "average waiting steps\n(%s)" % t, f1range, rf2range, waitStepsMinMax)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
# Copyright (C) 2008-2022 German Aerospace Center (DLR) and others.
# This program and the accompanying materials are made available under the
# terms of the Eclipse Public License 2.0 which is available at
# https://www.eclipse.org/legal/epl-2.0/
# This Source Code may also be made available under the following Secondary
# Licenses when the conditions for such availability set forth in the Eclipse
# Public License 2.0 are satisfied: GNU General Public License, version 2
# or later which is available at
# https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
# SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later

# @file    runner.py
# @author  Jakob Erdmann
# @date    2017-01-23


from __future__ import print_function
from __future__ import absolute_import
import os
import sys

if "SUMO_HOME" in os.environ:
    sys.path.append(os.path.join(os.environ["SUMO_HOME"], "tools"))

import traci  # noqa
import sumolib  # noqa

cmd = [sumolib.checkBinary('sumo'),
       '-n', 'input_net2.net.xml',
       '-r', 'input_routes.rou.xml',
       '--no-step-log',
       '--tripinfo-output', 'tripinfo.xml',
       '--tripinfo-output.write-unfinished',
       # '-S', '-Q',
       ]

ANGLE_UNDEF = traci.constants.INVALID_DOUBLE_VALUE
INVALID = traci.constants.INVALID_DOUBLE_VALUE

vehID = "ego"

traci.start(cmd)
traci.simulationStep()
x, y = traci.vehicle.getPosition(vehID)
for dy in [1, 1, 1, 2, 2, 2, 0, 0, 0, 1, 1, 1, 2, 2, 2, 0, 0, 0, -1, -1, -1, 1, 1, 1]:
    traci.vehicle.moveToXY(vehID, "", 0, x, y)
    traci.simulationStep()
    print("speed=%.2f waitingTime=%s" % (
        traci.vehicle.getSpeed(vehID),
        traci.vehicle.getWaitingTime(vehID)))
    y += dy
traci.close()
